#!/usr/bin/env python

"""
A Rectangle stimulus.

This module contains a class implementing a rectangle stimulus.

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


import pygame

from . import defaults
from ._visual import Visual
from .. import _internals

class Rectangle(Visual):
    """A class implementing a rectangle stimulus."""

    def __init__(self, size, colour=None, line_width=None,
                 corner_rounding=None, corner_anti_aliasing=None,
                 position=None):
        """Create a rectangle.

        Parameters
        ----------
        size : (int, int)
            size (width, height) of the rectangle
        colour : (int, int, int), optional
            colour of the rectangle
        line_width : int, optional
            line width in pixels; 0 will result in a filled rectangle,
            as does a value < 0 or >= min(size)
        corner_rounding : float or (float, float, float, float), optional
            radius of the corners in percent of 0.5 * min(size);
        corner_anti_aliasing : int, optional
            anti aliasing parameter for rounded corners
            (good anti_aliasing with 10)
        position : (int, int), optional
            position of the stimulus

        """

        if position is None:
            position = defaults.rectangle_position
        Visual.__init__(self, position=position)
        self._size = size
        if colour is None:
            colour = defaults.rectangle_colour
        if colour is not None:
            self._colour = colour
        else:
            self._colour = _internals.active_exp.foreground_colour
        if line_width is None:
            line_width = defaults.rectangle_line_width
        elif line_width < 0 or line_width >= min(self._size):
            line_width = 0
        self._line_width = line_width
        if corner_rounding is None:
            corner_rounding = defaults.rectangle_corner_rounding
        elif corner_rounding < 0 or corner_rounding > 100:
            raise AttributeError("corner_rounding must be >= 0 and < 100!")
        self._corner_rounding = corner_rounding
        if corner_anti_aliasing is not None:
            self._corner_anti_aliasing = corner_anti_aliasing
        else:
            self._corner_anti_aliasing = defaults.rectangle_corner_anti_aliasing

    _getter_exception_message = "Cannot set {0} if surface exists!"

    @property
    def size(self):
        """Getter for size."""
        return self._size

    @size.setter
    def size(self, value):
        """Setter for size."""

        if self.has_surface:
            raise AttributeError(Rectangle._getter_exception_message.format(
                "size"))
        else:
            self._size = value

    @property
    def colour(self):
        """Getter for colour."""
        return self._colour

    @colour.setter
    def colour(self, value):
        """Setter for colour."""

        if self.has_surface:
            raise AttributeError(Rectangle._getter_exception_message.format(
                "colour"))
        else:
            self._colour = value

    @property
    def line_width(self):
        """Getter for line_width."""
        return self._line_width

    @line_width.setter
    def line_width(self, value):
        """Setter for line_width."""

        if self.has_surface:
            raise AttributeError(Rectangle._getter_exception_message.format(
                "line_width"))
        else:
            self._line_width = value

    @property
    def corner_rounding(self):
        """Getter for corner_rounding."""
        return self._corner_rounding

    @corner_rounding.setter
    def corner_rounding(self, value):
        """Setter for corner_rounding."""

        if self.has_surface:
            raise AttributeError(Rectangle._getter_exception_message.format(
                "corner_rounding"))
        else:
            self._corner_rounding = value

    @property
    def corner_anti_aliasing(self):
        """Getter for corner_anti_aliasing."""
        return self._corner_anti_aliasing

    @corner_anti_aliasing.setter
    def corner_anti_aliasing(self, value):
        """Setter for corner_anti_aliasing."""

        if self.has_surface:
            raise AttributeError(Rectangle._getter_exception_message.format(
                "anti_aliasing"))
        else:
            self._anti_aliasing = value

    def _create_surface(self):
        """Create the surface of the stimulus."""

        def create_rounded_rectangle(rectangle_size, inverted=False):
            """Helper function to create (uncoloured) rounded rectangles."""

            if self._corner_anti_aliasing > 0:
                aa_scaling = (self._corner_anti_aliasing / 5.0) + 1
            else:
                aa_scaling = 1

            if inverted:
                scaled_size = [x * aa_scaling for x in rectangle_size]
                surface = pygame.surface.Surface(scaled_size,
                                             pygame.SRCALPHA).convert_alpha()
                rounding = self._corner_rounding * 0.5 * \
                           min(surface.get_size()) // 100
                size = [2 * rounding,
                        2 * rounding]
                tmp = pygame.surface.Surface(scaled_size)
                rect1 = pygame.surface.Surface(
                    (surface.get_size()[0] - 2 * rounding,
                     surface.get_size()[1]))
                rect2 = pygame.surface.Surface(
                    (surface.get_size()[0],
                     surface.get_size()[1] - 2 * rounding))
                tmp.fill((0, 0, 0))
                rect1.fill((255, 255, 255))
                rect2.fill((255, 255, 255))
                tmp.blit(rect1, (rounding, 0))
                tmp.blit(rect2, (0, rounding))
                circle = pygame.surface.Surface(
                    size)
                pygame.draw.ellipse(circle, (255, 255, 255),
                                    pygame.Rect((0, 0), size))
                tmp.blit(circle, (0, 0), special_flags=pygame.BLEND_RGB_MAX)
                tmp.blit(circle, (surface.get_size()[0] - rounding * 2, 0),
                         special_flags=pygame.BLEND_RGB_MAX)
                tmp.blit(circle, (0,
                                  surface.get_size()[1] - rounding * 2),
                         special_flags=pygame.BLEND_RGB_MAX)
                tmp.blit(circle, (surface.get_size()[0] - rounding * 2,
                                  surface.get_size()[1] - rounding * 2),
                         special_flags=pygame.BLEND_RGB_MAX)
                tmp.set_colorkey((255, 255, 255))
                surface.blit(tmp, (0, 0))

                if aa_scaling != 1:
                    surface = pygame.transform.smoothscale(
                        surface, (int(surface.get_size()[0] / aa_scaling),
                                  int(surface.get_size()[1] / aa_scaling)))

            else:
                surface = pygame.surface.Surface(
                    rectangle_size, pygame.SRCALPHA).convert_alpha()
                rounding = self._corner_rounding * 0.5 * min(surface.get_size()) // 100
                size = [(2 * rounding) * aa_scaling,
                        (2 * rounding) * aa_scaling]
                rect1 = pygame.surface.Surface(
                    (surface.get_size()[0] - 2 * rounding,
                     surface.get_size()[1]))
                rect2 = pygame.surface.Surface(
                    (surface.get_size()[0],
                     surface.get_size()[1] - 2 * rounding))
                rect1.fill((0, 0, 0))
                rect2.fill((0, 0, 0))
                surface.blit(rect1, (rounding, 0))
                surface.blit(rect2, (0, rounding))
                circle = pygame.surface.Surface(
                    size, pygame.SRCALPHA).convert_alpha()
                pygame.draw.ellipse(circle, (0, 0, 0),
                                    pygame.Rect((0, 0), size))

                if aa_scaling != 1:
                    circle = pygame.transform.smoothscale(
                        circle, (int(size[0] / aa_scaling),
                                 int(size[1] / aa_scaling)))

                surface.blit(circle, (0, 0))
                surface.blit(circle, (surface.get_size()[0] - rounding * 2,
                                      0))
                surface.blit(circle, (0,
                                      surface.get_size()[1] - rounding * 2))
                surface.blit(circle, (surface.get_size()[0] - rounding * 2,
                                      surface.get_size()[1] - rounding * 2))

            return surface

        if self._line_width == 0:
            if self._corner_rounding == 0:
                surface = pygame.surface.Surface(
                    self._size, pygame.SRCALPHA).convert_alpha()
                surface.fill(self._colour)
            else:
                surface = create_rounded_rectangle(self._size)
                surface.fill(self._colour, special_flags=pygame.BLEND_RGB_MAX)

        else:
            if self._line_width % 2 == 0:
                surface_size = [x + self._line_width for x in self._size]
                hole_size = [x - self._line_width for x in self._size]
            else:
                surface_size = [x + self._line_width - 1 for x in self._size]
                hole_size = [x - (self._line_width + 1) for x in self._size]
            surface = pygame.surface.Surface(
                surface_size,  #[x + self._line_width for x in self._size],
                pygame.SRCALPHA).convert_alpha()
            if self._corner_rounding == 0:
                surface.fill((0, 0, 0))
                hole = pygame.surface.Surface(
                    hole_size,  #[x - self._line_width for x in self._size],
                    pygame.SRCALPHA).convert_alpha()
                surface.blit(hole, (self._line_width, self._line_width),
                             special_flags=pygame.BLEND_RGBA_MIN)
                surface.fill(self._colour, special_flags=pygame.BLEND_RGB_MAX)
            else:
                surface = create_rounded_rectangle(
                    surface_size)  #[x + self._line_width for x in self._size])
                hole = create_rounded_rectangle(
                    hole_size,  #[x - self._line_width for x in self._size],
                    inverted=True)
                surface.blit(hole, (self._line_width, self._line_width),
                             special_flags=pygame.BLEND_RGBA_MIN)
                surface.fill(self._colour, special_flags=pygame.BLEND_RGB_MAX)

        return surface

    def is_point_inside(self, point_xy):
        """"OBSOLETE METHOD: Please use 'overlapping_with_position'."""

        raise DeprecationWarning("is_point_inside is an obsolete method. Please use overlapping_with_position")

    @staticmethod
    def _demo(exp=None):
        if exp is None:
            from .. import control
            control.set_develop_mode(True)
            control.defaults.event_logging = 0
            exp_ = control.initialize()
        rect = Rectangle((20, 200), colour=(255, 0, 255))
        rect.present()
        if exp is None:
            exp_.clock.wait(1000)



"""A gamepad.

This module contains a class implementing a pygame gamepad.

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


import time
from types import FunctionType

import pygame

from .. import _internals
from ..misc._timer import get_time
from ._keyboard import Keyboard
from  ._input_output import Input, Output


pygame.joystick.init()


class GamePad(Input, Output):
    """A class for creating gamepad/joystick input."""

    @staticmethod
    def get_gampad_count():
        """Get the number of gamepads/joysticks connected to the system."""

        return pygame.joystick.get_count()

    def __init__(self, gamepad_id, track_button_events=True,
                 track_motion_events=False):
        """Create a gamepad/joystick input.

        Parameters
        ----------
        gamepad_id : int
            id of the gamepad
        track_button_events : bool, optional
            Track button events (default=True)
        track_motion_events : bool, optional
            Track motion events (default=False)

        """

        if not _internals.active_exp.is_initialized:
            raise RuntimeError(
                "Cannot create GamePad before expyriment.initialize()!")
        Input.__init__(self)
        Output.__init__(self)
        self.track_button_events = track_button_events
        self.track_motion_events = track_motion_events
        self._joystick = pygame.joystick.Joystick(gamepad_id)
        self._joystick.init()

    @property
    def track_button_events(self):
        """Getter for track_button_events."""

        return self._track_button_events

    @track_button_events.setter
    def track_button_events(self, value):
        """Setter for track_button_events.

        Switch on/off the processing of button events.

        """

        self._track_button_events = value
        if value:
            pygame.event.set_allowed(pygame.JOYBUTTONDOWN)
            pygame.event.set_allowed(pygame.JOYBUTTONUP)
        else:
            pygame.event.set_blocked(pygame.JOYBUTTONDOWN)
            pygame.event.set_blocked(pygame.JOYBUTTONUP)

    @property
    def track_motion_events(self):
        """Getter for track_motion_events."""

        return self._track_motion_events

    @track_motion_events.setter
    def track_motion_events(self, value):
        """Setter for track_motion_events.

        Switch on/off the processing of motion events.

        """

        self._track_motion_events = value
        if value:
            pygame.event.set_allowed(pygame.JOYAXISMOTION)
            pygame.event.set_allowed(pygame.JOYBALLMOTION)
            pygame.event.set_allowed(pygame.JOYHATMOTION)
        else:
            pygame.event.set_blocked(pygame.JOYAXISMOTION)
            pygame.event.set_blocked(pygame.JOYBALLMOTION)
            pygame.event.set_blocked(pygame.JOYHATMOTION)

    @property
    def id(self):
        """Getter for id."""

        return self._joystick.get_id()

    @property
    def name(self):
        """Getter for name."""

        return self._joystick.get_name()

    @property
    def joystick(self):
        """Getter for joystick."""

        return self._joystick

    def get_numaxes(self):
        """Get the number of axes."""

        return self._joystick.get_numaxes()

    def get_axis(self, axis):
        """Get current axis state.

        Parameters
        ----------
        axis : int
            axis to get the current state from

        """

        pygame.event.pump()
        return self._joystick.get_axis(axis)

    def get_numballs(self):
        """Get the number of balls."""

        return self._joystick.get_numballs()

    def get_ball(self, ball):
        """Get current ball state.

        Parameters
        ----------
        ball : int
            ball to get the current state from

        """

        pygame.event.pump()
        return self._joystick.get_ball(ball)

    def get_numbuttons(self):
        """Get the number of buttons."""

        return self._joystick.get_numbuttons()

    def get_button(self, button):
        """Get current button state.

        Parameters
        ----------
        button : int
            button to get the current state from

        """

        pygame.event.pump()
        return self._joystick.get_button(button)

    def get_numhats(self):
        """Get the number of hats."""

        return self._joystick.get_numhats()

    def get_hat(self, hat):
        """Get current hat state.

        Parameters
        ----------
        hat : int
            hat to get the current state from

        """

        pygame.event.pump()
        return self._joystick.get_hat(hat)


    def clear(self):
        """Clear gamepad events from cue."""

        pygame.event.clear(pygame.JOYBUTTONDOWN)
        pygame.event.clear(pygame.JOYBUTTONUP)
        pygame.event.clear(pygame.JOYAXISMOTION)
        pygame.event.clear(pygame.JOYBALLMOTION)
        pygame.event.clear(pygame.JOYHATMOTION)
        if self._logging:
            _internals.active_exp._event_file_log("GamePad,cleared", 2)

    def wait_press(self, buttons=None, duration=None, callback_function=None,
                   process_control_events=True):
        """Wait for gamepad button press.

        Returns the found button and the reaction time.

        Parameters
        ----------
        buttons : int or list, optional
            specific buttons to wait for
        duration : int, optional
            maximal time to wait in ms
        callback_function : function, optional
            function to repeatedly execute during waiting loop
        process_control_events : bool, optional
            process ``io.Keyboard.process_control_keys()`` and
            ``io.Mouse.process_quit_event()`` (default = True)

        Returns
        -------
        button : int
            button _id of the pressed button
        rt : int
            reaction time in ms

        Notes
        ------
        This will also by default process control events (quit and pause).
        Thus, keyboard events will be cleared from the cue and cannot be
        received by a Keyboard().check() anymore!

        See Also
        --------
        expyriment.design.Experiment.register_wait_callback_function

        """

        if _internals.skip_wait_methods:
            return None, None
        start = get_time()
        rt = None
        _button = None
        self.clear()
        if buttons is None:
            buttons = list(range(self.get_numbuttons()))
        try:
            buttons = list(buttons)
        except:
            buttons = [buttons]
        done = False
        while not done:
            if isinstance(callback_function, FunctionType):
                rtn_callback = callback_function()
                if isinstance(rtn_callback, _internals.CallbackQuitEvent):
                    _button = rtn_callback
                    rt = int((get_time() - start) * 1000)
                    done = True
            if _internals.active_exp.is_initialized:
                rtn_callback = _internals.active_exp._execute_wait_callback()
                if isinstance(rtn_callback, _internals.CallbackQuitEvent):
                    _button = rtn_callback
                    rt = int((get_time() - start) * 1000)
                    done = True
                if process_control_events:
                    if _internals.active_exp.mouse.process_quit_event() or \
                       _internals.active_exp.keyboard.process_control_keys():
                        done = True
            for button in buttons:
                if self.get_button(button):
                    _button = button
                    rt = int((get_time() - start) * 1000)
                    done = True
                    break
                if _button is not None:
                    done = True
                    break
                if duration:
                    if int((get_time() - start) * 1000) >= duration:
                        done = True
                        break

            time.sleep(0.0005)

        if self._logging:
            _internals.active_exp._event_file_log(
                            "Gamepad,received,{0},wait_press".format(_button))
        return _button, rt



# -*- coding: utf-8 -*-
"""Colour class.

This module contains a class implementing an RGB colour.

"""
from __future__ import absolute_import, print_function, division
from builtins import *


__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


import colorsys

from . import round


# The named colours are the 140 HTML colour names:
#    see https://www.w3schools.com/colors/colors_names.asp
_colours = {
    'aliceblue':                (240, 248, 255),
    'antiquewhite':             (250, 235, 215),
    'aqua':                     (0, 255, 255),
    'aquamarine':               (127, 255, 212),
    'azure':                    (240, 255, 255),
    'beige':                    (245, 245, 220),
    'bisque':                   (255, 228, 196),
    'black':                    (0, 0, 0),
    'blanchedalmond':           (255, 235, 205),
    'blue':                     (0, 0, 255),
    'blueviolet':               (138, 43, 226),
    'brown':                    (165, 42, 42),
    'burlywood':                (222, 184, 135),
    'cadetblue':                (95, 158, 160),
    'chartreuse':               (127, 255, 0),
    'chocolate':                (210, 105, 30),
    'coral':                    (255, 127, 80),
    'cornflowerblue':           (100, 149, 237),
    'cornsilk':                 (255, 248, 220),
    'crimson':                  (220, 20, 60),
    'cyan':                     (0, 255, 255),
    'darkblue':                 (0, 0, 139),
    'darkcyan':                 (0, 139, 139),
    'darkgoldenrod':            (184, 134, 11),
    'darkgray':                 (169, 169, 169),
    'darkgreen':                (0, 100, 0),
    'darkkhaki':                (189, 183, 107),
    'darkmagenta':              (139, 0, 139),
    'darkolivegreen':           (85, 107, 47),
    'darkorange':               (255, 140, 0),
    'darkorchid':               (153, 50, 204),
    'darkred':                  (139, 0, 0),
    'darksalmon':               (233, 150, 122),
    'darkseagreen':             (143, 188, 143),
    'darkslateblue':            (72, 61, 139),
    'darkslategray':            (47, 79, 79),
    'darkturquoise':            (0, 206, 209),
    'darkviolet':               (148, 0, 211),
    'deeppink':                 (255, 20, 147),
    'deepskyblue':              (0, 191, 255),
    'dimgray':                  (105, 105, 105),
    'dodgerblue':               (30, 144, 255),
    'firebrick':                (178, 34, 34),
    'floralwhite':              (255, 250, 240),
    'forestgreen':              (34, 139, 34),
    'fuchsia':                  (255, 0, 255),
    'gainsboro':                (220, 220, 220),
    'ghostwhite':               (248, 248, 255),
    'gold':                     (255, 215, 0),
    'goldenrod':                (218, 165, 32),
    'gray':                     (128, 128, 128),
    'green':                    (0, 128, 0),
    'greenyellow':              (173, 255, 47),
    'honeydew':                 (240, 255, 240),
    'hotpink':                  (255, 105, 180),
    'indianred':                (205, 92, 92),
    'indigo':                   (75, 0, 130),
    'ivory':                    (255, 255, 240),
    'khaki':                    (240, 230, 140),
    'lavender':                 (230, 230, 250),
    'lavenderblush':            (255, 240, 245),
    'lawngreen':                (124, 252, 0),
    'lemonchiffon':             (255, 250, 205),
    'lightblue':                (173, 216, 230),
    'lightcoral':               (240, 128, 128),
    'lightcyan':                (224, 255, 255),
    'lightgoldenrodyellow':     (250, 250, 210),
    'lightgray':                (211, 211, 211),
    'lightgreen':               (144, 238, 144),
    'lightpink':                (255, 182, 193),
    'lightsalmon':              (255, 160, 122),
    'lightseagreen':            (32, 178, 170),
    'lightskyblue':             (135, 206, 250),
    'lightslategray':           (119, 136, 153),
    'lightsteelblue':           (176, 196, 222),
    'lightyellow':              (255, 255, 224),
    'lime':                     (0, 255, 0),
    'limegreen':                (50, 205, 50),
    'linen':                    (250, 240, 230),
    'magenta':                  (255, 0, 255),
    'maroon':                   (128, 0, 0),
    'mediumaquamarine':         (102, 205, 170),
    'mediumblue':               (0, 0, 205),
    'mediumorchid':             (186, 85, 211),
    'mediumpurple':             (147, 112, 219),
    'mediumseagreen':           (60, 179, 113),
    'mediumslateblue':          (123, 104, 238),
    'mediumspringgreen':        (0, 250, 154),
    'mediumturquoise':          (72, 209, 204),
    'mediumvioletred':          (199, 21, 133),
    'midnightblue':             (25, 25, 112),
    'mintcream':                (245, 255, 250),
    'mistyrose':                (255, 228, 225),
    'moccasin':                 (255, 228, 181),
    'navajowhite':              (255, 222, 173),
    'navy':                     (0, 0, 128),
    'oldlace':                  (253, 245, 230),
    'olive':                    (128, 128, 0),
    'olivedrab':                (107, 142, 35),
    'orange':                   (255, 165, 0),
    'orangered':                (255, 69, 0),
    'orchid':                   (218, 112, 214),
    'palegoldenrod':            (238, 232, 170),
    'palegreen':                (152, 251, 152),
    'paleturquoise':            (175, 238, 238),
    'palevioletred':            (219, 112, 147),
    'papayawhip':               (255, 239, 213),
    'peachpuff':                (255, 218, 185),
    'peru':                     (205, 133, 63),
    'pink':                     (255, 192, 203),
    'plum':                     (221, 160, 221),
    'powderblue':               (176, 224, 230),
    'purple':                   (128, 0, 128),
    'red':                      (255, 0, 0),
    'rosybrown':                (188, 143, 143),
    'royalblue':                (65, 105, 225),
    'saddlebrown':              (139, 69, 19),
    'salmon':                   (250, 128, 114),
    'sandybrown':               (250, 164, 96),
    'seagreen':                 (46, 139, 87),
    'seashell':                 (255, 245, 238),
    'sienna':                   (160, 82, 45),
    'silver':                   (192, 192, 192),
    'skyblue':                  (135, 206, 235),
    'slateblue':                (106, 90, 205),
    'slategray':                (112, 128, 144),
    'snow':                     (255, 250, 250),
    'springgreen':              (0, 255, 127),
    'steelblue':                (70, 130, 180),
    'tan':                      (210, 180, 140),
    'teal':                     (0, 128, 128),
    'thistle':                  (216, 191, 216),
    'tomato':                   (255, 99, 71),
    'turquoise':                (64, 224, 208),
    'violet':                   (238, 130, 238),
    'wheat':                    (245, 222, 179),
    'white':                    (255, 255, 255),
    'whitesmoke':               (245, 245, 245),
    'yellow':                   (255, 255, 0),
    'yellowgreen':              (154, 205, 50),
}


class Colour(object):
    """Implements a class representing an RGB colour."""

    @staticmethod
    def get_colour_names():
        """Get a dictionary of all known colour names."""

        from collections import OrderedDict
        return OrderedDict(sorted(_colours.items(), key=lambda t: t[0]))

    @staticmethod
    def is_rgb(value):
        """Check for valid RGB tuple value.

        Parameters
        ----------
        value : iterable of length 3 (e.g. [255, 0, 0])
            the value to be checked

        Returns
        -------
        valid : bool
            whether the value is valid or not

        """

        if not len(value) == 3:
            return False
        elif False in [isinstance(x, int) for x in value]:
            return False
        elif False in [0 <= x <= 255 for x in value]:
            return False
        else:
            return True

    @staticmethod
    def is_name(value):
        """Check for valid colour name value.

        Parameters
        ----------
        value : str (e.g. "red")
            the value to be checked

        Returns
        -------
        valid : bool
            whether the value is valid or not

        """

        if not value in _colours.keys():
            return False
        else:
            return True

    @staticmethod
    def is_hex(value):
        """Check for valid Hex triplet value.

        Parameters
        ----------
        value : string (e.g. "#FF0000")
            the value to be checked

        Returns
        -------
        valid : bool
            whether the value is valid or not

        """

        if not len(value) == 3:
            return False
        elif False in [isinstance(x, int) for x in value]:
            return False
        elif False in [0 <= x <= 100 for x in value]:
            return False
        else:
            return True

    @staticmethod
    def is_hsv(value):
        """Check for valid HSV tuple value.

        Parameters
        ----------
        value : iterable of length 3 (e.g. [0, 100, 100])
            the value to be checked

        Returns
        -------
        valid : bool
            whether the value is valid or not

        """

        if not len(value) == 3:
            return False
        elif False in [isinstance(x, int) for x in value]:
            return False
        elif not 0 <= value[0] <= 360:
            return False
        elif False in [0 <= x <= 100 for x in value[1:]]:
            return False
        else:
            return True

    @staticmethod
    def is_hsl(value):
        """Check for valid HSL tuple value.

        Parameters
        ----------
        value : iterable of length 3 (e.g. [0, 100, 50])
            the value to be checked

        Returns
        -------
        valid : bool
            whether the value is valid or not

        """

        return Colour.is_hsv(value)

    @staticmethod
    def is_colour(value):
        """Check for valid colour value.

        Parameters
        ----------
        value : any type
            the value to be checked

        Returns
        -------
        valid : bool
            whether the value is valid or not

        """

        return Colour.is_rgb(value) or\
            Colour.is_name(value) or \
            Colour.is_hex(value) or \
            Colour.is_hlv(value) or\
            Colour.is_hls(value)

    def __init__(self, colour):
        """Create an RGB colour.

        Parameters
        ----------
        colour : list or tuple or str
            the colour to be created as either an RGB tuple (e.g.[255, 0, 0]),
            a Hex triplet (e.g. "#FF0000") or a colour name (e.g. "red").

        Notes
        -----
        All methods in Expyriment that have a colour parameter require RGB
        colours. This class also allows RGB colours to be defined via HSV/HSL
        values (hue [0-360], saturation [0-100], value/lightness [0-100]).To
        do so, use the hsv or hls property.

        """

        if Colour.is_rgb(colour):
            self.rgb = colour
        elif Colour.is_hex(colour):
            self.hex = colour
        elif Colour.is_name(colour):
            self.name = colour
        else:
            raise ValueError("'{0}' is not a valid colour!".format(colour) + \
            "\nUse RGB tuple, Hex triplet or colour name.")

    def __str__(self):
        return "Colour(red={0}, green={1}, blue={2})".format(self._rgb[0],
                                                             self._rgb[1],
                                                             self._rgb[2])

    def __eq__(self, other):
        return self._rgb == other._rgb

    def __ne__(self, other):
        return self._rgb != other._rgb

    def __getitem__(self, i):
        return self._rgb[i]

    def __len__(self):
        return len(self._rgb)

    @property
    def rgb(self):
        """Getter for colour in RGB format [red, green, blue]."""
        return self._rgb

    @rgb.setter
    def rgb(self, value):
        """Setter for colour in RGB format [red, green, blue]."""

        if Colour.is_rgb(value):
            self._rgb = tuple(value)
        else:
            raise ValueError("'{0}' is not a valid RGB colour!".format(value))

    @property
    def hex(self):
        """Getter for colour in Hex format "#RRGGBB"."""

        return '#{:02X}{:02X}{:02X}'.format(self._rgb[0],
                                            self._rgb[1],
                                            self._rgb[2])
    @hex.setter
    def hex(self, value):
        """Setter for colour in Hex format "#RRGGBB"."""

        if Colour.is_hex(value):
            c = value.lstrip("#")
            self._rgb = tuple(int(c[i:i + 2], 16) for i in (0, 2, 4))
        else:
            raise ValueError("'{0}' is not a valid Hex colour!".format(value))

    @property
    def name(self):
        """Getter for colour name (if available)."""

        for name, rgb in _colours.items():
            if rgb == self.rgb:
                return name
        return None

    @name.setter
    def name(self, value):
        """Setter for colour name."""

        if Colour.is_name(value):
            self._rgb = _colours[value.lower()]
        else:
            raise ValueError("'{0}' is not a valid colour name!".format(value))

    @property
    def hsv(self):
        """Getter for colour in HSV format [hue, saturation, value]."""

        hsv = colorsys.rgb_to_hsv(*divide(self.rgb, 255.0))
        rtn = list(multiply([hsv[0]], 360))
        rtn.extend(multiply(hsv[1:], 100))
        return rtn

    @hsv.setter
    def hsv(self, value):
        """Setter for colour in HSV format [hue, saturation, value]."""

        if Colour.is_hsv(value):
            hsv = list(divide([value[0]], 360))
            hsv.extend(divide(value[1:], 100))
            self._rgb = multiply(colorsys.hsv_to_rgb(*hsv), 255)
        else:
            raise ValueError("'{0}' is not a valid HSV colour!".format(value))

    @property
    def hsl(self):
        """Getter for colour in HSL format [hue, saturation, lightness]."""

        hsl = colorsys.rgb_to_hls(*divide(self.rgb, 255.0))
        rtn = list(multiply([hsl[0]], 360))
        rtn.extend(multiply(hsl[1:], 100))
        return rtn

    @hsl.setter
    def hsl(self, value):
        """Setter for colour in HSL format [hue, saturation, lightness]."""

        if Colour.is_hsv(value):
            hsl = list(divide([value[0]], 360))
            hsl.extend(divide(value[1:], 100))
            self._rgb = multiply(colorsys.hls_to_rgb(*hsl), 255)
        else:
            raise ValueError("'{0}' is not a valid HSL colour!".format(value))


# Helper functions
def multiply(v, d):
    return tuple(map(lambda x:int(round(x*d)), v))

def divide(v, d):
    return tuple(map(lambda x:x/float(d), v))



"""A high-resolution monotonic timer

This module provides a high-resolution timer via the function get_time()

Thanks to Luca Filippin for the code examples.

Credits and references:
      http://stackoverflow.com/questions/1205722/how-do-i-get-monotonic-time-durations-in-python
      http://stackoverflow.com/questions/1824399/get-mach-absolute-time-uptime-in-nanoseconds-in-python
      https://mail.python.org/pipermail/python-dev/2009-October/093173.html
"""
from __future__ import absolute_import, print_function, division
from builtins import *


__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''

# use perf_counter if it exists and works
try:
    from time import perf_counter
    def get_time():
        """Get high-resolution time stamp (float) """
        return perf_counter()

    get_time()

except:

    try:
        import ctypes
    except:
        ctypes = None  # Does not exist on Android
    import os
    from sys import platform

    _fallback_time_module = False

    if platform == 'darwin':
        # MAC
        try:
            class _TimeBase(ctypes.Structure):
                _fields_ = [
                    ('numer', ctypes.c_uint),
                    ('denom', ctypes.c_uint)
                ]

            _libsys_c = ctypes.CDLL('/usr/lib/system/libsystem_c.dylib')
            _libsys_kernel = ctypes.CDLL('/usr/lib/system/libsystem_kernel.dylib')
            _mac_abs_time = _libsys_c.mach_absolute_time
            _mac_timebase_info = _libsys_kernel.mach_timebase_info
            _time_base = _TimeBase()
            if (_mac_timebase_info(ctypes.pointer(_time_base)) != 0):
                _fallback_time_module = True

            def get_time():
                """Get high-resolution monotonic time stamp (float) """
                _mac_abs_time.restype = ctypes.c_ulonglong
                return float(_mac_abs_time()) * _time_base.numer / (_time_base.denom * 1e9)
            get_time()
        except:
            _fallback_time_module = True

    elif platform.startswith('linux'):
        # real OS
        _CLOCK_MONOTONIC = 4  # actually CLOCK_MONOTONIC_RAW see <linux/time.h>

        try:
            class _TimeSpec(ctypes.Structure):
                _fields_ = [
                    ('tv_sec', ctypes.c_long),
                    ('tv_nsec', ctypes.c_long)
                ]

            _librt = ctypes.CDLL('librt.so.1', use_errno=True)
            _clock_gettime = _librt.clock_gettime
            _clock_gettime.argtypes = [ctypes.c_int, ctypes.POINTER(_TimeSpec)]

            def get_time():
                """Get high-resolution monotonic time stamp (float) """
                t = _TimeSpec()
                if _clock_gettime(_CLOCK_MONOTONIC, ctypes.pointer(t)) != 0:
                    errno_ = ctypes.get_errno()
                    raise OSError(errno_, os.strerror(errno_))
                return t.tv_sec + t.tv_nsec * 1e-9
            get_time()
        except:
            _fallback_time_module = True

    elif platform == 'win32':
        # win32. Code adapted from the psychopy.core.clock source code.
        try:
            _fcounter = ctypes.c_int64()
            _qpfreq = ctypes.c_int64()
            ctypes.windll.Kernel32.QueryPerformanceFrequency(ctypes.byref(_qpfreq))
            _qpfreq = float(_qpfreq.value)
            _winQPC = ctypes.windll.Kernel32.QueryPerformanceCounter

            def get_time():
                """Get high-resolution monotonic time stamp (float) """
                _winQPC(ctypes.byref(_fcounter))
                return  _fcounter.value / _qpfreq
            get_time()
        except:
            _fallback_time_module = True
    else:
        # Android or something else
        _fallback_time_module = True


    if _fallback_time_module:
        import time
        if platform == 'win32':
            timer_used = "clock"
            def get_time():
                """Get high-resolution time stamp (float) """
                return time.clock()

        else:
            timer_used = "time"
            def get_time():
                """Get high-resolution time stamp (float) """
                return time.time()

        print("Warning: Failed to initialize monotonic timer. Python's "
                             "time.{}() function will be used.".format(
                        timer_used))



"""The io package.

This package contains several classes and functions that implement
input and output interfaces.

See also expyriment.io.extras for more io.

"""

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


from . import defaults
from ._screen import Screen
from ._keyboard import Keyboard
from ._mouse import Mouse
from ._files import InputFile, OutputFile, DataFile, EventFile
from ._parallelport import ParallelPort
from ._serialport import SerialPort
from ._gamepad import GamePad
from ._eventbuttonbox import EventButtonBox
from ._streamingbuttonbox import StreamingButtonBox
from ._triggerinput import TriggerInput
from ._markeroutput import MarkerOutput
from ._textinput import TextInput
from ._textmenu import TextMenu
from ._touchscreenbuttonbox import TouchScreenButtonBox



"""
A text input box.

This module contains a class implementing a text input box for user input.

"""
from __future__ import absolute_import, print_function, division
from builtins import (ascii, bytes, chr, dict, filter, hex, input,
                      int, map, next, oct, pow, range, round,
                      str, super, zip)  # without open, because
                       # pygame.font.Font needs old file object under PY2


__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


import pygame
try:
    import android
except ImportError:
    android = None

try:
    import android.show_keyboard as android_show_keyboard
    import android.hide_keyboard as android_hide_keyboard
except ImportError:
    android_show_keyboard = android_hide_keyboard = None

from . import defaults
from .. import _internals, stimuli
from ..misc import find_font, unicode2byte, constants, \
                 numpad_digit_code2ascii
from .._internals import CallbackQuitEvent
from ._input_output import Input


class TextInput(Input):
    """A class implementing a text input box."""

    def __init__(self, message="", position=None, character_filter=None,
                 length=None, message_text_size=None, message_colour=None,
                 message_font=None, message_bold=None, message_italic=None,
                 message_right_to_left=None, user_text_size=None,
                 user_text_bold=None, user_text_font=None,
                 user_text_colour=None, user_right_to_left=None,
                 background_colour=None, frame_colour=None, gap=None,
                 screen=None, background_stimulus=None, **kwargs):

        """Create a text input box.

        Notes
        -----
        This stimulus is not optimized for timing accurate presentation!

        Parameters
        ----------
        message : str, optional
            message to show
        position : (int, int), optional
            position of the TextInput canvas
        length : int, optional
            the length of the text input frame in number of characters
        character_filter : list, optional
            list of character codes to filter for
        message_text_size : int, optional
            text size of the message
        message_colour : (int, int, int), optional
            text colour of the message
        message_font : str, optional
            text font of the message
        message_bold : bool, optional
            True if message text should be bold
        message_italic : bool, optional
            True if message text should be italic
        message_right_to_left : bool, optional
            whether or not the message text should be presented right-to-left
        user_text_size : int, optional
            text size of the user input
        user_text_font : str, optional
            text font of the user input
        user_text_colour : (int, int ,int), optional
            text colour of the user input
        user_text_bold : bool, optional
            True if user text should be bold
        user_right_to_left : bool, optional
            whether or not the user text should be presented right-to-left
        background_colour : (int, int, int), optional
        frame_colour : (int, int, int)
            colour of the frame
        gap : int, optional
            gap between message and user input
        screen : io.Screen, optional
            screen to present on
        background_stimulus : visual Expyriment stimulus, optional
            The background stimulus is a second stimulus that will be presented
            together with the TextInput. For both stimuli overlap TextInput
            will appear on top of the background_stimulus

        """

        if not _internals.active_exp.is_initialized:
            raise RuntimeError(
                "Cannot create TextInput before expyriment.initialize()!")
        Input.__init__(self)
        self._message = message
        if position is not None:
            self._position = position
        else:
            self._position = defaults.textinput_position
        if character_filter is not None:
            self._character_filter = character_filter
        else:
            self._character_filter = defaults.textinput_character_filter
        if "ascii_filter" in kwargs:
            self._character_filter = kwargs["ascii_filter"]
        if length is not None:
            self._length = length
        else:
            self._length = defaults.textinput_length
        if message_text_size is None:
            message_text_size = defaults.textinput_message_text_size
        if message_text_size is not None:
            self._message_text_size = message_text_size
        else:
            self._message_text_size = _internals.active_exp.text_size
        if message_colour is None:
            message_colour = defaults.textinput_message_colour
        if message_colour is not None:
            self._message_colour = message_colour
        else:
            self._message_colour = _internals.active_exp.foreground_colour
        if message_font is None:
            message_font = defaults.textinput_message_font
        if message_font is None:
            message_font = _internals.active_exp.text_font
        if message_font is None:
            message_font = "FreeSans"
        self._message_font = find_font(message_font)
        try:
            with open(self._message_font, 'rb') as f:
                pygame.font.Font(f, 10)
        except:
            raise IOError("Font '{0}' not found!".format(message_font))
        if message_bold is not None:
            self._message_bold = message_bold
        else:
            self._message_bold = defaults.textinput_message_bold
        if message_italic is not None:
            self._message_italic = message_italic
        else:
            self._message_italic = defaults.textinput_message_italic
        if message_right_to_left is not None:
            self._message_right_to_left = message_right_to_left
        else:
            self._message_right_to_left = defaults.textinput_message_right_to_left
        if user_text_size is None:
            user_text_size = defaults.textinput_user_text_size
        if user_text_size is not None:
            self._user_text_size = user_text_size
        else:
            self._user_text_size = _internals.active_exp.text_size

        if user_text_bold is not None:
            self._user_text_bold = user_text_bold
        else:
            self._user_text_bold = defaults.textinput_user_text_bold
        if user_text_font is None:
            user_text_font = defaults.textinput_user_text_font
        if user_text_font is None:
            user_text_font = _internals.active_exp.text_font
        if user_text_font is None:
            user_text_font = "FreeSans"
        self._user_text_font = find_font(user_text_font)
        try:
            with open(self._user_text_font, 'rb') as f:
                pygame.font.Font(f, 10)
        except:
            raise IOError("Font '{0}' not found!".format(user_text_font))
        if user_text_colour is None:
            user_text_colour = defaults.textinput_user_text_colour
        if user_text_colour is not None:
            self._user_text_colour = user_text_colour
        else:
            self._user_text_colour = _internals.active_exp.foreground_colour
        if user_right_to_left is not None:
            self._user_right_to_left = user_right_to_left
        else:
            self._user_right_to_left = defaults.textinput_user_right_to_left
        if background_colour is None:
            background_colour = \
                defaults.textinput_background_colour
        if background_colour is not None:
            self._background_colour = background_colour
        else:
            self._background_colour = \
                _internals.active_exp.background_colour
        if frame_colour is None:
            frame_colour = defaults.textinput_frame_colour
        if frame_colour is not None:
            self._frame_colour = frame_colour
        else:
            self._frame_colour = _internals.active_exp.foreground_colour
        if gap is not None:
            self._gap = gap
        else:
            self._gap = defaults.textinput_gap
        if screen is not None:
            self._screen = screen
        else:
            self._screen = _internals.active_exp.screen
        if background_stimulus is not None:
            if background_stimulus.__class__.__bases__[0] == stimuli._visual.Visual:
                self._background_stimulus = background_stimulus
            else:
                raise TypeError("{0} ".format(type(background_stimulus)) +
                                     "is not a valid background stimulus. " +
                                     "Use an expyriment visual stimulus.")
        else:
            self._background_stimulus = None

        self._user = []
        self._user_text_surface_size = None
        self._max_size = None
        self._message_surface_size = None
        self._canvas = None
        self._canvas_size = None

    @property
    def message(self):
        """Getter for message"""
        return self._message

    @property
    def position(self):
        """Getter for position"""
        return self._position

    @property
    def character_filter(self):
        """Getter for character filter"""

        return self._character_filter

    @character_filter.setter
    def character_filter(self, value):
        """Getter for character filter"""

        self._character_filter = value

    @property
    def message_text_size(self):
        """Getter for message_text_size"""
        return self._message_text_size

    @property
    def length(self):
        """Getter for length"""
        return self._length

    @property
    def message_colour(self):
        """Getter for message_colour"""
        return self._message_colour

    @property
    def message_font(self):
        """Getter for message_font"""
        return self._message_font

    @property
    def message_bold(self):
        """Getter for message_bold"""
        return self._message_bold

    @property
    def message_italic(self):
        """Getter for message_italic"""
        return self._message_italic

    @property
    def message_right_to_left(self):
        """Getter for message_right_to_left"""
        return self._message_right_to_left

    @property
    def user_text_size(self):
        """Getter for user_text_size"""
        return self._user_text_size

    @property
    def user_text_bold(self):
        """Getter for user_text_bold"""
        return self._user_text_bold

    @property
    def user_text_font(self):
        """Getter for user_text_font"""
        return self._user_text_font

    @property
    def user_text_colour(self):
        """Getter for user_text_colour"""
        return self._user_text_colour

    @property
    def user_right_to_left(self):
        """Getter for user_right_to_left"""
        return self._user_right_to_left

    @property
    def background_colour(self):
        """Getter for background_colour"""
        return self._background_colour

    @property
    def frame_colour(self):
        """Getter for frame_colour"""
        return self._frame_colour

    @property
    def gap(self):
        """Getter for gap"""
        return self._gap

    @property
    def screen(self):
        """Getter for screen"""
        return self._screen

    @property
    def background_stimulus(self):
        """Getter for background_stimulus"""
        return self._background_stimulus

    def _get_key(self, process_control_events):
        """Get a key press."""

        while True:
            rtn_callback = _internals.active_exp._execute_wait_callback()
            if isinstance(rtn_callback, CallbackQuitEvent):
                return rtn_callback, None

            events = pygame.event.get(pygame.KEYDOWN)
            for event in events:
                if process_control_events:
                    if _internals.active_exp.keyboard.process_control_keys(event) or \
                            _internals.active_exp.mouse.process_quit_event():
                        self._create()
                        self._update()
                        return None, None
                return event.key, event.unicode

    def _create(self):
        """Create the input box."""

        tmp = stimuli.TextLine(text=self._length * "X",
                               text_font=self.user_text_font,
                               text_size=self.user_text_size,
                               text_bold=self.user_text_bold)
        stimuli._stimulus.Stimulus._id_counter -= 1
        self._max_size = tmp.surface_size
        text = self._message
        if self._message_right_to_left:
            text = self._message[::-1]
        message_text = stimuli.TextLine(
            text=text, text_font=self.message_font,
            text_size=self.message_text_size, text_bold=self.message_bold,
            text_italic=self.message_italic, text_colour=self.message_colour,
            background_colour=self._background_colour)
        stimuli._stimulus.Stimulus._id_counter -= 1
        self._message_surface_size = message_text.surface_size

        self._canvas = stimuli.Canvas(size=(
            max(self._max_size[0] + 12, self._message_surface_size[0]),
            self._message_surface_size[1] + self._max_size[1] + self._gap + 5),
            colour=self._background_colour, position=self._position)

        #self._canvas = expyriment.stimuli.BlankScreen()

        stimuli._stimulus.Stimulus._id_counter -= 1
        self._canvas._set_surface(self._canvas._get_surface())
        self._canvas_size = self._canvas.surface_size
        pygame.draw.rect(self._canvas._get_surface(), self._background_colour,
                         (self._canvas_size[0] // 2 - self._max_size[0] // 2 - 6,
                          self._message_surface_size[1] + self._gap,
                          self._max_size[0] + 12, self._max_size[1] + 5), 0)
        pygame.draw.rect(self._canvas._get_surface(), self._frame_colour,
                         (self._canvas_size[0] // 2 - self._max_size[0] // 2 - 6,
                          self._message_surface_size[1] + self._gap,
                          self._max_size[0] + 12, self._max_size[1] + 5), 1)
        if len(self._message) != 0:
            if self._message_right_to_left:
                self._canvas._get_surface().blit(
                    message_text._get_surface(),
                    (self._canvas.surface_size[0] -
                    self._message_surface_size[0], 0))
            else:
                self._canvas._get_surface().blit(
                    message_text._get_surface(),
                    (self._canvas.surface_size[0] // 2 -
                    self._message_surface_size[0] // 2, 0))
        background = stimuli.BlankScreen(
            colour=self._background_colour)
        if self._background_stimulus is not None:
            self._background_stimulus.plot(background)
        self._canvas.plot(background)
        background.present()
        background.present()  # for flipping with double buffer
        background.present()  # for flipping with tripple buffer

    def _update(self):
        """Update the input box."""

        user_canvas = stimuli.Canvas(
            size=self._max_size, colour=self._background_colour)
        stimuli._stimulus.Stimulus._id_counter -= 1
        user_canvas._set_surface(user_canvas._get_surface())
        user_canvas_size = user_canvas.surface_size
        offset = 2 + user_canvas_size[1] % 2
        user_canvas.position = (self._canvas.absolute_position[0],
                                self._canvas.absolute_position[1] +
                                self._canvas_size[1] // 2 -
                                user_canvas_size[1] // 2 -
                                self._message_surface_size[1] -
                                self._gap - offset)
        text = "".join(self._user)
        if self._user_right_to_left:
            text = "".join(self._user)[::-1]
        user_text = stimuli.TextLine(
            text=text,
            text_font=self.user_text_font, text_size=self.user_text_size,
            text_bold=self.user_text_bold, text_colour=self.user_text_colour,
            background_colour=self.background_colour)
        stimuli._stimulus.Stimulus._id_counter -= 1
        self._user_text_surface_size = user_text.surface_size
        if self._user_right_to_left:
            user_canvas._get_surface().blit(
                user_text._get_surface(),
                (user_canvas.surface_size[0] -
                 self._user_text_surface_size[0], 2))
        else:
            user_canvas._get_surface().blit(user_text._get_surface(), (0, 2))
        user_canvas.present(clear=False)

    def get(self, default_input="", process_control_events=True):
        """Get input from user.

        Notes
        -----
        This displays and updates the input box automatically. Pressing ENTER
        returns the user input.

        Parameters
        ----------
        default_input : str, optional
            default input in the textbox
        process_control_events : bool, optional
            process ``io.Keyboard.process_control_keys()`` and
            ``io.Mouse.process_quit_event()`` (default = True)

        Returns
        -------
        text_input: str or None
            returns the entered text string. If get() is interrupted by a
            CallbackQuitEvent from a registered wait-callback-function get()
            returns None.

        See Also
        --------
        expyriment.design.Experiment.register_wait_callback_function

        """

        if android_show_keyboard is not None:
            android_show_keyboard()
        self._user = []
        for char in default_input:
            self._user.append(char)
        self._create()
        self._update()
        if self._character_filter is None:
            filter = list(range(0, 256)) + constants.K_ALL_KEYPAD_DIGITS
        else:
            filter = self._character_filter

        while True:
            inkey, string = self._get_key(process_control_events)
            if isinstance(inkey, CallbackQuitEvent):
                return None
            elif inkey == pygame.K_BACKSPACE:
                self._user = self._user[0:-1]
            elif inkey == pygame.K_RETURN or inkey == pygame.K_KP_ENTER:
                break
            elif inkey not in (pygame.K_LCTRL, pygame.K_RCTRL, pygame.K_TAB):
                if not self._user_text_surface_size[0] >= self._max_size[0]:
                    if android is not None:
                        if inkey in filter:
                            if inkey in constants.K_ALL_KEYPAD_DIGITS:
                                inkey = numpad_digit_code2ascii(inkey)
                            self._user.append(chr(inkey))
                    else:
                        if inkey in constants.K_ALL_KEYPAD_DIGITS:
                            self._user.append(chr(numpad_digit_code2ascii(
                                inkey)))
                        elif string and ord(string) in filter:
                            self._user.append(string)
            self._update()
        got = "".join(self._user)
        if self._logging:
            _internals.active_exp._event_file_log(
                "TextInput,entered,{0}".format(got))
        if android_hide_keyboard is not None:
            android_hide_keyboard()
        return got


    @staticmethod
    def _demo(exp=None):
        if exp is None:
            from .. import control
            control.set_develop_mode(True)
            control.defaults.event_logging = 0
            exp_ = control.initialize()
        textinput = TextInput(message="Subject Number:",
                              message_colour=(160, 70, 250),
                              user_text_size=30,
                              user_text_colour=(255, 150, 50),
                              frame_colour=(70, 70, 70))
        print(textinput.get())



# -*- coding: utf-8 -*-
"""The expyriment testsuite.

This module contains several functions to test the machine expyriment is
running on.

"""
from __future__ import absolute_import, print_function, division
from builtins import *
from past.utils import old_div

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


import os

import pygame
try:
    import OpenGL.GL as ogl
except:
    ogl = None

from . import defaults, initialize, end
from .. import stimuli, io, _internals, design
import expyriment

from .. import misc
from ..misc import constants, statistics, list_fonts, unicode2byte
from ..misc._timer import get_time
from ..design import randomize

def _make_graph(x, y, colour):
    """Make the graph."""

    graph = stimuli.Canvas(size=(max(x) * 3 + 10, max(y) * 3 + 10))
    for counter in range(len(x)):
        dot = stimuli.Dot(radius=1, colour=colour)
        dot.position = (x[counter] * 3 - graph.size[0] // 2 + 5,
                        y[counter] * 3 - graph.size[1] // 2 + 5)
        dot.plot(graph)
    return graph

def _histogram(data):
    """Returns the hist of the data (list of numbers) as dict and
    as string representation

    """

    hist = {}
    for x in data: # make histogram
        x = int(x)
        if x in hist:
            hist[x] += 1
        else:
            hist[x] = 1
    #make string representation
    hist_str = ""
    cnt = 0
    str1 = None
    for x in range(min(hist.keys()), max(hist.keys())+1):
        if str1 is None:
            str1 = "dRT: "
            str2 = "  n: "
        str1 += "%4d" % x
        if x in hist:
            value = hist[x]
        else:
            value = 0
        str2 += "%4d" % value
        if x % 10 == 0:
            hist_str += str1 + "\n" + str2 + "\n\n"
            str1 = None

    if str1 is not None:
        hist_str += str1 + "\n" + str2 + "\n\n"
    return hist, hist_str

def _stimulus_timing(exp):
    """Test the timing of stimulus presentation."""

    def _test1():
        info = """This will test the visual stimulus presentation timing specifics of your system.
During the test, you will see two squares on the screen.
After the test, you will be asked to indicate which (if any) of those two squares were flickering.

[Press RETURN to continue]"""

        text = stimuli.TextScreen("Visual stimulus presentation test", info)
        #y = []
        #for x in [16, 32, 48, 64]:
        #    y.extend([x, x, x, x, x, x, x, x, x, x, x, x, x, x, x, ])
        #graph1 = _make_graph(range(60), y, [0, 255, 0])
        #y = range(80)
        #graph2 = _make_graph(range(60), y, [255, 0, 0])
        #graph1.position = (-200, -100)
        #graph2.position = (200, -100)
        while True:
            text.present()
        #graph1.present(clear=False, update=False)
        #graph2.present(clear=False)
            key, rt_ = exp.keyboard.wait([constants.K_RETURN])
            if key is not None:
                break
        message = stimuli.TextScreen("Running", "Please wait...")
        message.present()
        message.present()
        message.present()
        c1 = stimuli.Canvas((400, 400))
        c2 = stimuli.Canvas((400, 400))
        c3 = stimuli.Canvas((400, 400))
        frame1 = stimuli.Rectangle((100, 100), position=(-100, 0))
        frame2 = stimuli.Rectangle((100, 100), position=(100, 0))
        bg = stimuli.Rectangle((90, 90), colour=exp.background_colour)
        bg.plot(frame1)
        bg.plot(frame2)
        frame1.plot(c1)
        frame2.plot(c2)
        frame1.plot(c3)
        frame2.plot(c3)
        c1.preload()
        c2.preload()
        c3.preload()
        c1.present(clear=False)
        c2.present(clear=False)
        c3.present(clear=False)

        s1 = stimuli.Circle(1, colour=exp.background_colour)
        s2 = stimuli.Circle(1, colour=exp.background_colour)
        s1.preload()
        s2.preload()
        to_do_time = list(range(0, 60)) * 3
        randomize.shuffle_list(to_do_time)
        actual_time = []
        for x in to_do_time:
            s1.present(clear=False)
            start = get_time()
            exp.clock.wait(x)
            s2.present(clear=False)
            actual_time.append((get_time() - start) * 1000)
            exp.clock.wait(randomize.rand_int(30, 60))

        # determine refresh_rate
        tmp = []
        for _x in range(100):
            start = get_time()
            s1.present(clear=False)
            tmp.append(get_time() - start)
            start = get_time()
            s2.present(clear=False)
            tmp.append(get_time() - start)
        refresh_rate = 1000 // (statistics.mean(tmp) * 1000)

        #text = stimuli.TextScreen("Results", "[Press RETURN to continue]")
        #graph = _make_graph(to_do_time, actual_time, [150, 150, 150])
        #graph.position = (0, -100)
        #text.present(update=False)
        #graph.present(clear=False)
        #exp.keyboard.wait([constants.K_RETURN])
        #text = stimuli.TextScreen(
        #    "Which picture looks most similar to the results?",
        #    "[Press LEFT or RIGHT arrow key]")
        #y = []
        #for x in [16, 32, 48, 64]:
        #    y.extend([x, x, x, x, x, x, x, x, x, x, x, x, x, x, x, ])
        #graph1 = _make_graph(range(60), y, [0, 255, 0])
        #y = range(80)
        #graph2 = _make_graph(range(60), y, [255, 0, 0])
        #graph1.position = (-200, -100)
        #graph2.position = (200, -100)
        #text.present(update=False)
        #graph1.present(clear=False, update=False)
        #graph2.present(clear=False)
        #key, _rt = exp.keyboard.wait([constants.K_LEFT,
        #                             constants.K_RIGHT])
        #if key == constants.K_LEFT:
        #    response1 = "Steps"
        #elif key == constants.K_RIGHT:
        #    response1 = "Line"
        #else:
        #    response1 = None


        # show histogram of presentation delays
        def expected_delay(presentation_time, refresh_rate):
            refresh_time = old_div(1000.0, refresh_rate)
            return refresh_time - (presentation_time % refresh_time)
        # delay = map(lambda x: x[1]- x[0], zip(to_do_time, actual_time))
        unexplained_delay = [x[1]- x[0] - expected_delay(x[0], refresh_rate) for x in zip(to_do_time, actual_time)]
        hist, hist_str = _histogram(unexplained_delay)
        inaccuracies = []
        delayed_presentations = 0
        for key in list(hist.keys()):
            inaccuracies.extend([key % max(1, (1000 // refresh_rate))] * hist[key])
            if key != 0:
                delayed_presentations += hist[key]
        inaccuracy = int(misc.round(old_div(sum(inaccuracies), float(len(inaccuracies)))))
        delayed = misc.round(100 * delayed_presentations/180.0, 2)

        respkeys = {constants.K_F1:0, constants.K_F2:1, constants.K_F3:2,
                    constants.K_0:0, constants.K_1:1, constants.K_2:2}
        text = stimuli.TextScreen(
            "How many of the two squares were flickering?",
            "[Press 0 (or F1), 1 (or F2), 2 (or F3)]")
        while True:
            text.present()
            key, _rt = exp.keyboard.wait(respkeys)
            if key is not None:
                break
        response = respkeys[key]

        info = stimuli.TextScreen("Results", "")
        if int(misc.round(refresh_rate))  < 50 or int(misc.round(refresh_rate)) > 120:
            results1_colour = [255, 0, 0]
        elif int(misc.round(refresh_rate)) != 60:
            results1_colour = [255, 255, 0]
        else:
            results1_colour = [0, 255, 0]
        results1 = stimuli.TextScreen("",
                    "Estimated Screen Refresh Rate:     {0} Hz (~ every {1} ms)\n\n".format(
                        int(misc.round(refresh_rate)), int(old_div(1000.0, refresh_rate))),
                    text_font="freemono", text_size = 16, text_bold=True,
                    text_justification=0, text_colour=results1_colour, position=(0, 40))
        results2 = stimuli.TextScreen("",
                    "Detected Framebuffer Pages:        {0}\n\n".format(response+1),
                    text_font="freemono", text_size = 16, text_bold=True,
                    text_justification=0, position=(0, 20))
        if inaccuracy != 0:
            results3_colour = [255, 0, 0]
        else:
            results3_colour = [0, 255, 0]
        results3 = stimuli.TextScreen("",
                    "Average Reporting Inaccuracy:      {0} ms\n\n".format(inaccuracy),
                    text_font="freemono", text_size = 16, text_bold=True,
                    text_justification=0, text_colour=results3_colour, position=(0, -20))
        if delayed > 10:
            results4_colour = [255, 0, 0]
        elif 10 > delayed > 1:
            results4_colour = [255, 255, 0]
        else:
            results4_colour = [0, 255, 0]
        results4 = stimuli.TextScreen("",
                    "Unexplained Presentation Delays:   {0} %\n\n\n".format(delayed),
                    text_font="freemono", text_size = 16, text_bold=True,
                    text_justification=0, text_colour=results4_colour, position=(0, -40))
        results5 = stimuli.TextScreen("",
                    hist_str,
                    text_font="freemono", text_size = 16, text_bold=True,
                    text_justification=0, position=(0, -100))
        results1.plot(info)
        results2.plot(info)
        results3.plot(info)
        results4.plot(info)
        results5.plot(info)
        info2 = stimuli.TextLine("[Press RETURN to continue]", position=(0, -160))
        info2.plot(info)
        while True:
            info.present()
            key, rt_ = exp.keyboard.wait([constants.K_RETURN])
            if key is not None:
                break
        return to_do_time, actual_time, refresh_rate, inaccuracy, delayed, response



#     def _test2():
#         info = """This will test if stimulus presentation can be synchronized to the refreshrate of the screen.
# A good result is a fast, constant and smooth flickering without any distortions (e.g. horizontal stripes, tearing).
# The estimated refreshrate should resemble your actual screen refreshrate (common refreshrates are between 40 and 240 Hz).
#
# [Press RETURN to continue]"""
#
#         text = stimuli.TextScreen("Stimulus presentation test (2)", info)
#         text.present()
#         exp.keyboard.wait([constants.K_RETURN])
#         black = stimuli.BlankScreen(colour=constants.C_BLACK)
#         black.preload()
#         white = stimuli.BlankScreen(colour=constants.C_WHITE)
#         white.preload()
#         times = []
#         black.present()
#         for _x in range(100):
#             start = get_time()
#             black.present()
#             times.append(get_time() - start)
#             start = get_time()
#             white.present()
#             times.append(get_time() - start)
#         refresh_rate = 1000 / (statistics.mean(times) * 1000)
#         info = """Your estimated refresh rate is {0} Hz.
#
# [Press RETURN to continue]
# """.format(refresh_rate)
#         text = stimuli.TextScreen("Results", info)
#         text.present()
#         exp.keyboard.wait([constants.K_RETURN])
#         text = stimuli.TextScreen(
#             "Was the flickering fast, constant and smooth, without any distortions?",
#             "[Press Y or N]")
#         text.present()
#         key, _rt = exp.keyboard.wait([constants.K_y,
#                                       constants.K_n])
#         if key == constants.K_y:
#             response2 = "Yes"
#         elif key == constants.K_n:
#             response2 = "No"
#
#         return refresh_rate, response2

#     def _test2():
#         info = """This will test the video card's settings for multiple buffers and page flipping.
# If none of the following squares are constantly blinking, page flipping is not activated and buffer contents are copied.
# If only the left square is constantly blinking, page flipping is activated and a double buffer is used.
# If additionally the right square is constantly blinking, page flipping is activated and a tripple buffer is used.
#
# [Press RETURN to continue]"""
#
#         text = stimuli.TextScreen("Visual stimulus presentation test (2)", info)
#         text.present()
#         exp.keyboard.wait([constants.K_RETURN])
#         c1 = stimuli.Canvas((400, 400))
#         c2 = stimuli.Canvas((400, 400))
#         c3 = stimuli.Canvas((400, 400))
#         frame1 = stimuli.Rectangle((100, 100), position=(-100, 0))
#         frame2 = stimuli.Rectangle((100, 100), position=(100, 0))
#         bg = stimuli.Rectangle((90, 90), colour=exp.background_colour)
#         bg.plot(frame1)
#         bg.plot(frame2)
#         frame1.plot(c1)
#         frame2.plot(c2)
#         frame1.plot(c3)
#         frame2.plot(c3)
#         c1.preload()
#         c2.preload()
#         c3.preload()
#         c1.present()
#         c2.present()
#         c3.present()
#         for _x in range(50):
#             d = stimuli.Dot(1, colour=(1, 1, 1))
#             d.present(clear=False)
#             exp.clock.wait(100)
#         text = stimuli.TextScreen(
#             "How many squares were constantly blinking?",
#             "[Press 0, 1 or 2]")
#         text.present()
#         key, _rt = exp.keyboard.wait([constants.K_0,
#                                       constants.K_1,
#                                       constants.K_2])
#         if key == constants.K_0:
#             response3 = 0
#         elif key == constants.K_1:
#             response3 = 1
#         elif key == constants.K_2:
#             response3 = 2
#
#         return (response3,)

    return _test1()


def _audio_playback(exp):
    """Test the audio playback"""

    info = """This will test the audio playback. A test tone will be played.

[Press RETURN to continue]
"""
    text = stimuli.TextScreen("Audio playback test", info)
    while True:
        text.present()
        key, rt_ = exp.keyboard.wait([constants.K_RETURN])
        if key is not None:
            break
    exp.screen.clear()
    exp.screen.update()
    a = stimuli.Tone(duration=1000)
    a.present()
    exp.clock.wait(1000)
    text = stimuli.TextScreen("Did you hear the tone?", "[Press Y or N]")
    while True:
        text.present()
        key, _rt = exp.keyboard.wait([constants.K_y,
                                      constants.K_n])
        if key is not None:
            break
    if key == constants.K_y:
        response = "Yes"
    elif key == constants.K_n:
        response = "No"

    return response

def _font_viewer(exp):
    all_fonts = list(list_fonts().keys())

    def info_screen():
        stimuli.TextScreen(heading="Expyriment Font Viewer",
            text="""
arrow keys left/right -- Switch font type
arrow keys up/down    -- Switch font size
                  i   -- Switch italic
                  b   -- Switch bold
                  c   -- Change text
                  h   -- Help
               return -- Quit


                 [Touch screen]
click left/right side --  Switch font type
click up/down side    --  Switch font size
click center          --  Quit
               """,
            text_font="freemono", text_bold=True,
            text_justification=0).present()
        exp.keyboard.wait()


    default_text = u"""The quick brown fox jumps over the lazy dog.
ABCDEFGHIJKLMNOPQRSTUVWXYZ ÄÖÜ
abcdefghijklmnopqrstuvwxyz äöü
1234567890.:,;ßéèê(*!?')"""
    text = default_text
    size = 14
    font_id = 0
    italic = False
    bold = False
    quest = io.TextInput(message="Please enter text: (Keep empty for default text)", length=35)
    mouse = io.Mouse(show_cursor=True)

    bs = (exp.screen.size[0] // 3.5, exp.screen.size[1] // 3.5)

    # rects center, left, right, top, button]
    cl = (20, 20, 20)
    rects = [stimuli.Rectangle(size=bs, position=[0, 0], colour=cl),
             stimuli.Rectangle(size=bs, position=[old_div((bs[0] - exp.screen.size[0]), 2.2), 0], colour=cl),
             stimuli.Rectangle(size=bs, position=[old_div((exp.screen.size[0] - bs[0]), 2.2), 0], colour=cl),
             stimuli.Rectangle(size=bs, position=[0, old_div((bs[1] - exp.screen.size[1]), 2.2)], colour=cl),
             stimuli.Rectangle(size=bs, position=[0, old_div((exp.screen.size[1] - bs [1]), 2.2)], colour=cl)]
    rect_key_mapping = [constants.K_RETURN, constants.K_LEFT, constants.K_RIGHT,
                        constants.K_UP, constants.K_DOWN]

    info_screen()
    while True:
        font_str = all_fonts[font_id]
        font_description = "font '{0}', size {1}".format(font_str, size)
        if italic:
            font_description += ", italic"
        if bold:
            font_description += ", bold"

        canvas = stimuli.BlankScreen()
        for r in rects:
            r.plot(canvas)
        try:
            stimuli.TextScreen(
                heading=font_description,
                text=text,
                text_font=font_str, text_size=size,
                text_justification=0,
                text_italic=italic,
                text_bold=bold,
                text_colour=(255, 255, 255)).plot(canvas)
        except:
            stimuli.TextLine(text="Sorry, I can't display the text with " +
                "{0}".format(font_description),
                text_colour=constants.C_EXPYRIMENT_ORANGE).plot(canvas)
        canvas.present()
        mouse.clear()
        exp.keyboard.clear()
        while True:
            key = exp.keyboard.check()
            if mouse.get_last_button_down_event() is not None:
                for cnt, r in enumerate(rects):
                    if r.overlapping_with_position(mouse.position):
                        key = rect_key_mapping[cnt]
                        break
            if key is not None:
                break

        if (key == constants.K_RETURN):
            break
        elif key == constants.K_UP:
            size += 2
        elif key == constants.K_DOWN:
            size -= 2
        elif key == constants.K_LEFT:
            font_id -= 1
            if font_id < 0:
                font_id = len(all_fonts) - 1
        elif key == constants.K_RIGHT:
            font_id += 1
            if font_id >= len(all_fonts):
                font_id = 0
        elif key == constants.K_i:
            italic = not(italic)
        elif key == constants.K_b:
            bold = not(bold)
        elif key == constants.K_c:
            text = quest.get()
            if len(text) <= 0:
                text = default_text
        else:
            info_screen()
    mouse.hide_cursor()


def _write_protocol(exp, results):
    """Write a protocol with all test results."""

    sorted_keys = list(results.keys())
    sorted_keys.sort()
    rtn = ""
    for key in sorted_keys:
        tabs = "\t" * (4 - int((len(key) + 1) // 8)) + "\t"
        try:
            rtn += key + ":" + tabs + results[key] + "\n"
        except TypeError:
            rtn += key + ":" + tabs + repr(results[key]) + "\n"

    filename = os.path.join(os.getcwd(), "test_suite_protocol.xpp")
    with open(filename, 'wb') as f:
        f.write(unicode2byte(rtn))
    while True:
        text = stimuli.TextScreen(
            "Saved as",
            '"' + filename + '"' + "\n\n[Press RETURN to continue]")
        text.present()
        key, rt_ = exp.keyboard.wait(constants.K_RETURN)
        if key is not None:
            break
    return []  # required for event loop

def _find_self_tests():
    classes = []
    method = []
    rtn = []
    namesspace = {}
    namesspace["expyriment"] = expyriment
    for module in ["expyriment.io", "expyriment.io.extras"]:
        exec("classes = dir({0})".format(module), namesspace)
        for cl in namesspace['classes']:
            if not cl.startswith("_") and not cl in ["False", "None", "True"]:
                exec("method = dir({0}.{1})".format(module, cl), namesspace)
                if "_self_test" in namesspace['method']:
                    rtn.append([module, cl])
    return rtn

def run_test_suite():
    """Run the Expyriment test suite."""

    # test imports
    from ..design import extras as _test1
    from ..stimuli import extras as _test2
    from ..io import extras as _test3
    from ..misc import extras as _test4
    from ..misc import get_system_info
    from .._internals import get_version

    quit_experiment = False
    if not _internals.active_exp.is_initialized:
        defaults.initialize_delay = 0
        defaults.event_logging = 0
        exp = design.Experiment()
        exp.testsuite = True
        initialize(exp)
        quit_experiment = True
    else:
        exp = _internals.active_exp

    # make menu and code for test functions
    test_functions = ['', '', '']
    menu = ["1) Visual stimulus presentation",
            "2) Auditory stimulus presentation",
            "3) Font Viewer"]
    for mod, cl in _find_self_tests():
        test_functions.append("rtn = {0}.{1}._self_test(exp)".format(mod, cl))
        menu.append("{0}) {1} test".format(len(test_functions), cl))

    menu.append("{0}) Write protocol".format(len(test_functions) + 1))
    menu.append("{0}) Quit".format(len(test_functions) + 2))
    test_functions.extend(['rtn = _write_protocol(exp, results)',
                            'go_on=False;rtn=[];'])

    background = stimuli.Canvas(size=[400, 600])
    pict = stimuli.Picture(constants.EXPYRIMENT_LOGO_FILE, position=(0, 255))
    pict.scale(0.35)
    pict.plot(background)

    v = stimuli.TextLine("Version {0}".format(get_version()), text_size=10,
                         text_colour=constants.C_EXPYRIMENT_PURPLE)
    v.move((0, 205))
    v.plot(background)
    results = get_system_info()

    try:
        import android
        mouse = io.Mouse(show_cursor=False)
    except ImportError:
        android = None
        mouse = None

    preselected_item = 0
    go_on = True
    while go_on:
        select = io.TextMenu("Test suite",
            menu, width=350, justification=0, text_size=18,
            background_stimulus=background, mouse=mouse).get(preselected_item)

        if select == 0:
            rtn = _stimulus_timing(exp)
            results["testsuite_visual_timing_to_do"] = rtn[0]
            results["testsuite_visual_timing_actual"] = rtn[1]
            results["testsuite_visual_sync_refresh_rate"] = str(rtn[2]) + " Hz"
            results["testsuite_visual_timing_inaccuracy"] = str(rtn[3]) + " ms"
            results["testsuite_visual_timing_delayed"] = str(rtn[4]) + " %"
            results["testsuite_visual_flipping_user"] = rtn[5]
            delay = [x[1]-x[0] for x in zip(results["testsuite_visual_timing_to_do"],
                               results["testsuite_visual_timing_actual"])]
            results["testsuite_visual_timing_delay_histogram"], _ = _histogram(delay)
            results["testsuite_visual_opengl"] = exp.screen.open_gl
            if ogl is not None and exp.screen.open_gl:
                results["testsuite_visual_opengl_vendor"] = ogl.glGetString(ogl.GL_VENDOR)
                results["testsuite_visual_opengl_renderer"] = ogl.glGetString(ogl.GL_RENDERER)
                results["testsuite_visual_opengl_version"] = ogl.glGetString(ogl.GL_VERSION)
                extensions = ogl.glGetString(ogl.GL_EXTENSIONS).decode().split(" ")
                if extensions[-1] == "":
                    extensions = extensions[:-1]
                results["testsuite_visual_opengl_extensions"] = extensions
            else:
                results["testsuite_visual_opengl_vendor"] = ""
                results["testsuite_visual_opengl_renderer"] = ""
                results["testsuite_visual_opengl_version"] = ""
                results["testsuite_visual_pygame_driver"] = ""
                results["testsuite_visual_opengl_extensions"] = ""
            results["testsuite_visual_pygame_driver"] = pygame.display.get_driver()
            results["testsuite_visual_pygame_screensize"] = exp.screen.size
            preselected_item = select + 1
        elif select == 1:
            results["testsuite_audio_user"] = _audio_playback(exp)
            try:
                results["testsuite_audio_frequency"] = str(pygame.mixer.get_init()[0]) + " Hz"
                results["testsuite_audio_bitdepth"] = str(abs(pygame.mixer.get_init()[1])) + " bit"
                results["testsuite_audio_channels"] = pygame.mixer.get_init()[2]
            except:
                results["testsuite_audio_frequency"] = ""
                results["testsuite_audio_bitdepth"] = ""
                results["testsuite_audio_channels"] = ""
            preselected_item = select + 1
        elif select == 2:
            _font_viewer(exp)
            preselected_item = select + 1
        else:
            namesspace = {}
            namesspace.update(globals())
            namesspace.update(locals())
            exec(test_functions[select], namesspace)
            rtn = namesspace['rtn']
            results.update(rtn)
            if 'go_on' in namesspace:
                go_on = namesspace['go_on']
            preselected_item = select + 1
            namesspace = None

    if quit_experiment:
        end(goodbye_delay=0, goodbye_text="Quitting test suite")
    else:
        exp.screen.clear()
        exp.screen.update()



"""
A touchscreen button box.

This module contains a class implementing a touchscreen button box.

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


from types import FunctionType

from .. import _internals, stimuli
from ._keyboard import Keyboard
from ..misc._timer import get_time
from .._internals import CallbackQuitEvent
from ._input_output import Input


class TouchScreenButtonBox(Input):
    """A class implementing a TouchScreenButtonBox."""

    def __init__(self, button_fields, stimuli=[], background_stimulus=None):
        """Initialize a touchscreen button box.

        Parameters
        ----------
        button_fields : visual Expyriment stimulus or list of stimuli
            The button fields defines the area on which a click action will be
            registered.
        stimuli : visual Expyriment stimulus or list of stimuli, optional
            Additional visual stimuli that will be presented together with the
            button fields. Stimuli are plotted on top of the button_fields.
        background_stimulus : visual Expyriment stimulus, optional
            The background stimulus on which the touschscreen button fields
            are presented. Importantly, background_stimulus has to have
            size of the screen.

        Notes
        -----
        Every visual Expyriment stimulus can serve as a touchscreen button
        field.
        If the TouchScreenButtonBox is presented, it can be checked for events
        using the methods 'check' and 'wait'.

        """

        Input.__init__(self)

        try:
            button_fields = list(button_fields)
        except:
            button_fields = [button_fields]
        try:
            stimuli = list(stimuli)
        except:
            stimuli = [stimuli]

        self._mouse = _internals.active_exp.mouse
        self._last_touch_position = None
        self._canvas = None
        self._button_fields = []
        self._stimuli = []
        self.background_stimulus = background_stimulus

        for x in button_fields:
            self.add_button_field(x)
        for x in stimuli:
            self.add_stimulus(x)

    def add_button_field(self, button_field):
        """Add a touchscreen button fields.

        Parameters
        ----------
        button_field : visual Expyriment stimulus

        """

        if not isinstance(button_field, stimuli._visual.Visual):
            raise TypeError("Button field has to be a visual Expyriment stimulus")
        self._button_fields.append(button_field)
        self._canvas = None

    def add_stimulus(self, stimulus):
        """Add additional stimulus

        The added stimulus will be presented together with the button fields.

        Parameters
        ----------
        stimulus : visual Expyriment stimulus

        """
        if not isinstance(stimulus, stimuli._visual.Visual):
            raise TypeError("Additional stimuli has to be a visual Expyriment stimulus")
        self._stimuli.append(stimulus)
        self._canvas = None

    @property
    def last_touch_position(self):
        """getter for the last touch position (x,y)"""
        return self._last_touch_position

    @property
    def button_field(self):
        """getter of button fields (list of visual Expyriment stimuli)"""
        return self._button_fields

    @property
    def stimuli(self):
        """getter of additional stimuli (list of visual Expyriment stimuli)"""
        return self._stimuli

    @property
    def background_stimulus(self):
        """Getter of background stimulus.

        Background stimulus, on which the button fields and the additional
        stimuli will be presented. (visual Expyriment stimuli)

        """
        return self._background_stimulus

    @background_stimulus.setter
    def background_stimulus(self, stimulus):
        """Setter background stimulus"""
        if stimulus is None:
            self._background_stimulus = stimuli.BlankScreen()
        elif not isinstance(stimulus, stimuli._visual.Visual):
            raise TypeError("Background stimulus has to be a " +
                            "visual Expyriment stimulus")
        else:
            self._background_stimulus = stimulus
        self._canvas = None

    def clear_event_buffer(self):
        """Clear the event buffer of the touchscreen/mouse input device."""

        if self._mouse is not None:
            self._mouse.clear()

    def create(self):
        """Create the touchscreen buttonbox.

        Prepares the button fields and additional stimuli for fast
        presentation.

        """

        self._canvas = self._background_stimulus.copy()
        print(self._canvas)
        if len(self._button_fields) < 1:
            raise RuntimeError("No button field defined!")
        for x in self._button_fields:
            x.plot(self._canvas)
        for x in self._stimuli:
            x.plot(self._canvas)
        self._canvas.preload()

    def destroy(self):
        if self._canvas is not None:
            self._canvas.unload()
        self._canvas = None

    def show(self):
        """Present touchscreen buttons."""

        if self._canvas is None:
            self.create()
        self._canvas.present()

    def check(self, button_fields=None):
        """Check if a button field is clicked.

        Parameters
        ----------
        button_fields : Expyriment stimulus or list of stimuli, optional
            The button fields that will be checked for.

        Returns
        -------
        pressed_button_field : Expyriment stimulus or list of stimuli, optional
            The button fields that will be checked for.
        touch_time : integer
            The time when the button was touched. Function might return delayed,
            because button field comparison (after touch) takes time. The
            return time is most accurate.


        Notes
        -----
        Don't forget to show the TouchScreenButtonBox.

        """

        if button_fields is not None:
            try:
                button_fields = list(button_fields)
            except:
                button_fields = [button_fields]

        pressed_button_field = None
        touch_time = None
        if self._mouse.get_last_button_down_event() is not None:
            touch_time = get_time()
            self._last_touch_position = self._mouse.position
            pressed_button_field = self._get_button_field(self._last_touch_position,
                    button_fields)

            if self._logging and pressed_button_field is not None:
                _internals.active_exp._event_file_log(
                                "{0},received, button press,check".format(
                                    self.__class__.__name__))
        return pressed_button_field, touch_time

    def _get_button_field(self, position, button_fields):
        """ helper function return the button field of the position"""
        if button_fields is None:
            button_fields = self._button_fields
        for bf in button_fields:
            if bf.overlapping_with_position(position):
                return bf
        return None

    def wait(self, duration=None, button_fields=None, callback_function=None,
                process_control_events=True):
        """Wait for a touchscreen button box click.

        Parameters
        ----------
        button_fields : Expyriment stimulus or list of stimuli, optional
            The button fields that will be checked for.
        duration : int, optional
            maximal time to wait in ms
        callback_function : function, optional
            function to repeatedly execute during waiting loop
        process_control_events : bool, optional
            process ``io.Keyboard.process_control_keys()`` and
            ``io.Mouse.process_quit_event()`` (default = True)

        Returns
        -------
        pressed_button_field : Expyriment stimulus or None
            the button field defined by a stimulus that has been pressed
        rt : int
            reaction time

        Notes
        -----
        This will also by default process control events (quit and pause).
        Thus, keyboard events will be cleared from the cue and cannot be
        received by a Keyboard().check() anymore!

        Don't forget to show the TouchScreenButtonBox.

        See Also
        --------
        expyriment.design.Experiment.register_wait_callback_function

        """

        if _internals.skip_wait_methods:
            return None, None
        start = get_time()
        self.clear_event_buffer()
        while True:
            if isinstance(callback_function, FunctionType):
                rtn_callback = callback_function()
                if isinstance(rtn_callback, CallbackQuitEvent):
                    return rtn_callback, int((get_time()-start)*1000)
            if _internals.active_exp.is_initialized:
                rtn_callback = _internals.active_exp._execute_wait_callback()
                if isinstance(rtn_callback, CallbackQuitEvent):
                    return rtn_callback, int((get_time()-start)*1000)
                if process_control_events:
                    if _internals.active_exp.mouse.process_quit_event() or \
                       _internals.active_exp.keyboard.process_control_keys():
                        pressed_button_field, rt = None, None
                        break
                else:
                    _internals.pump_pygame_events()
            pressed_button_field, touch_time = self.check(button_fields)
            if pressed_button_field is not None:
                rt = int((get_time()-start)*1000)
                break
            elif (duration is not None and int((get_time()-start)*1000)>=duration):
                pressed_button_field, rt = None, None
                break
        return pressed_button_field, rt



#!/usr/bin/env python

"""
A circle stimulus.

This module contains a class implementing a circle stimulus.

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


import math

from . import defaults
from ._ellipse import Ellipse


class Circle(Ellipse):
    """A class implementing a basic 2D circle."""

    def __init__(self, radius, colour=None, line_width=None, position=None,
                 anti_aliasing=None):
        """Create a circle.

        Parameters
        ----------
        radius : int
            radius of the circle
        colour : (int,int,int), optional
            colour of the circle
        line_width : int, optional
            line width in pixels betwee >= 0 and < radius,
            0 will result in a filled circle
        position : (int, int), optional
            position of the stimulus
        anti_aliasing : int, optional
            anti aliasing parameter (good anti_aliasing with 10)

        """

        self._radius = radius
        if position is None:
            position = defaults.circle_position
        if colour is None:
            colour = defaults.circle_colour
        if line_width is None:
            line_width = defaults.circle_line_width
        elif line_width < 0 or line_width >= self._radius:
            raise AttributeError("line_width must be >= 0 and < radius!")
        if anti_aliasing is not None:
            self._anti_aliasing = anti_aliasing
        else:
            self._anti_aliasing = defaults.circle_anti_aliasing
        Ellipse.__init__(self, [radius, radius], colour, line_width,
                         position, anti_aliasing)

    _getter_exception_message = "Cannot set {0} if surface exists!"

    @property
    def radius(self):
        """Getter for radius."""
        return self._radius

    @radius.setter
    def radius(self, value):
        """Setter for radius."""

        if self.has_surface:
            raise AttributeError(Circle._getter_exception_message.format(
                "radius"))
        else:
            self._radius = value

    def get_polar_coordiantes(self):
        """"OBSOLETE METHOD: Please use 'polar_coordinate'."""

        raise DeprecationWarning("get_polar_coordiantes is obsolete. " +
                                 "Please use the property polar_position")

    def set_polar_coordinates(self, radial, angle_in_degrees):
        """"OBSOLETE METHOD: Please use 'set_polar_coordinates'."""

        raise DeprecationWarning("set_polar_coordinates is obsolete. " +
                                 "Please use the property polar_position")

    def overlapping_with_circle(self, other, minimal_gap=0):
        """Return True if touching or overlapping with another circle.

        Parameters
        ----------
        other : expyriment.stimuli.Circle object
            other circle
        minimal_gap : int, optional
            minimum gap between two circle, small gaps will be treated as
            overlapping (default=0)

        Returns
        -------
        is_inside : bool

        """

        d = self.distance(other)
        return (d - minimal_gap <= other._radius + self._radius)

    def center_inside_circle(self, other):
        """Return True if the center is inside another circle.

        Parameters
        ----------
        other : expyriment.stimuli.Circle object
            other circle

        Returns
        -------
        is_inside : bool

        """

        d = self.distance(other)
        return (d <= other._radius)

    def inside_circle(self, other):
        """Return True if the whole circle is inside another circle.

        Parameters
        ----------
        other : expyriment.stimuli.Circle object
            other circle

        Returns
        -------
        is_inside : bool

        """

        d = self.distance(other)
        return (d <= other._radius - self._radius)


    @staticmethod
    def _demo(exp=None):
        if exp is None:
            from .. import control
            control.set_develop_mode(True)
            control.defaults.event_logging = 0
            exp_ = control.initialize()
        dot = Circle(radius=100, anti_aliasing=10)
        dot.present()
        if exp is None:
            exp_.clock.wait(1000)



#!/usr/bin/env python

"""
A text box stimulus.

This module contains a class implementing a text box stimulus.

"""
from __future__ import absolute_import, print_function, division
from builtins import (ascii, bytes, chr, dict, filter, hex, input,
                      int, map, next, oct, pow, range, round,
                      str, super, zip)  # without open, because
                       #  pygame.font.Font needs old file object under PY2


__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


import os
import re

import pygame

from . import defaults
from .. import _internals
from ..misc import find_font, unicode2byte, byte2unicode
from ._visual import Visual


# Keep track of open file handles and previously loaded font objects
open_filehandles = []
fonts = {}


class TextBox(Visual):
    """A class implementing a text box with wrapped text.

    This wraps a given multiline string into a formatted mutliline text box,
    strips of leading and trailing lines and gets rid of additional
    whitespaces at beginning of lines (indentation).

    """

    def __init__(self, text, size, position=None, text_font=None,
                 text_size=None, text_bold=None, text_italic=None,
                 text_underline=None, text_justification=None,
                 text_colour=None, background_colour=None,
                 do_not_trim_words=None):
        """Create a text box.

        Notes
        -----
        text_font can be both, a name or path to a font file!
        When text_font is a name, Expyriment will try to find a font that
        best matches the given name.
        If no matching font can be found, or if the given font file cannot be
        found, the Pygame system default will be used.
        In any case the value of the attribute text_font will always
        resemble the font that is actually in use!

        Parameters
        ----------
        text : str
            text to wrap
        size : (int, int)
            size of the text box
        position : (int, int), optional
            position of the stimulus
        text_font : str, optional
            text font to use as a name or as a path to a font file
        text_size : int, optional
            size of the text
        text_bold : bool, optional
            font should be bold
        text_italic : bool, optional
            font should be italic
        text_underline : bool, optional
            font should get an underline
        text_justification : int, optional
            text justification, 0 (left), 1 (center), 2 (right)
        text_colour : (int, int, int), optional
            colour of the text
        background_colour : (int, int, int), optional
            background colour
        do_not_trim_words: bool, optional
            if True, words that exceed the width of the text box
            will be not be trimmed and an exception is raise instead.
            default: False

        """

        pygame.font.init()

        if position is None:
            position = defaults.textbox_position
        Visual.__init__(self, position)
        self._text = text
        self._size = size
        if text_size is None:
            text_size = defaults.textbox_text_size
        if text_size is not None:
            self._text_size = text_size
        else:
            self._text_size = _internals.active_exp.text_size

        if text_font is None:
            text_font = defaults.textbox_text_font
        if text_font is None:
            text_font = _internals.active_exp.text_font
        if text_font is None:
            text_font = "FreeSans"
        self._text_font = find_font(text_font)
        try:
            with open(self.text_font, 'rb') as f:
                pygame.font.Font(f, 10)
        except:
            raise IOError("Font '{0}' not found!".format(text_font))
        if text_bold is not None:
            self._text_bold = text_bold
        else:
            self._text_bold = defaults.textbox_text_bold
        if text_italic is not None:
            self._text_italic = text_italic
        else:
            self._text_italic = defaults.textbox_text_italic
        if text_underline is not None:
            self._text_underline = text_underline
        else:
            self._text_underline = defaults.textbox_text_underline
        if text_justification is not None:
            self._text_justification = text_justification
        else:
            self._text_justification = defaults.textbox_text_justification
        if text_colour is not None:
            self._text_colour = text_colour
        else:
            if defaults.textbox_text_colour is not None:
                self._text_colour = defaults.textbox_text_colour
            else:
                self._text_colour = _internals.active_exp.foreground_colour
        if background_colour is not None:
            self._background_colour = background_colour
        else:
            self._background_colour = \
                defaults.textbox_background_colour
        if do_not_trim_words is not None:
            self._do_not_trim_words = do_not_trim_words
        else:
            self._do_not_trim_words = defaults.textbox_do_not_trim_words


    _getter_exception_message = "Cannot set {0} if surface exists!"

    @property
    def text(self):
        """Getter for text."""

        return self._text

    @text.setter
    def text(self, value):
        """Setter for text."""

        if self.has_surface:
            raise AttributeError(TextBox._getter_exception_message.format(
                "text"))
        else:
            self._text = value

    @property
    def text_font(self):
        """Getter for text_font."""

        return self._text_font

    @text_font.setter
    def text_font(self, value):
        """Setter for text_font."""

        if self.has_surface:
            raise AttributeError(TextBox._getter_exception_message.format(
                "text_font"))
        else:
            self._text_font = value

    @property
    def text_size(self):
        """Getter for text_size."""

        return self._text_size

    @text_size.setter
    def text_size(self, value):
        """Setter for text_size."""

        if self.has_surface:
            raise AttributeError(TextBox._getter_exception_message.format(
                "text_size"))
        else:
            self._text_size = value

    @property
    def text_bold(self):
        """Getter for text_bold."""

        return self._text_bold

    @text_bold.setter
    def text_bold(self, value):
        """Setter for text_bold."""

        if self.has_surface:
            raise AttributeError(TextBox._getter_exception_message.format(
                "text_bold"))
        else:
            self._text_bold = value

    @property
    def text_italic(self):
        """Getter for text_italic."""

        return self._text_italic

    @text_italic.setter
    def text_italic(self, value):
        """Setter for text_italic."""

        if self.has_surface:
            raise AttributeError(TextBox._getter_exception_message.format(
                "text_italic"))
        else:
            self._text_italic = value

    @property
    def text_underline(self):
        """Getter for text_underline."""

        return self._text_underline

    @text_underline.setter
    def text_underline(self, value):
        """Setter for text_underline."""

        if self.has_surface:
            raise AttributeError(TextBox._getter_exception_message.format(
                "text_underline"))
        else:
            self._text_underline = value

    @property
    def text_justification(self):
        """Getter for text_justification."""

        return self._text_justification

    @text_justification.setter
    def text_justification(self, value):
        """Setter for text_justification."""

        if self.has_surface:
            raise AttributeError(TextBox._getter_exception_message.format(
                "text_justification"))
        else:
            self._text_justification = value

    @property
    def text_colour(self):
        """Getter for text_colour."""

        return self._text_colour

    @text_colour.setter
    def text_colour(self, value):
        """Setter for text_colour."""

        if self.has_surface:
            raise AttributeError(TextBox._getter_exception_message.format(
                "text_colour"))
        else:
            self._text_colour = value

    @property
    def background_colour(self):
        """Getter for background_colour."""

        return self._background_colour

    @background_colour.setter
    def background_colour(self, value):
        """Setter for background_colour."""

        if self.has_surface:
            raise AttributeError(TextBox._getter_exception_message.format(
                "background_colour"))
        else:
            self._background_colour = value

    @property
    def size(self):
        """Getter for size."""
        return self._size

    @size.setter
    def size(self, value):
        """Setter for size."""

        if self.has_surface:
            raise AttributeError(TextBox._getter_exception_message.format(
                "size"))
        else:
            self._size = value

    def _create_surface(self):
        """Create the surface of the stimulus."""

        rect = pygame.Rect((0, 0), self.size)

        if os.path.isfile(self._text_font):
            # We preserve previously created fonts, so that we don't open the
            # same font over and over again.
            if (self._text_font, self._text_size) in fonts:
                _font = fonts[(self._text_font, self._text_size)]
            else:
                f = open(self._text_font, 'rb')
                open_filehandles.append(f)
                _font = pygame.font.Font(f, self._text_size)
                fonts[(self._text_font, self._text_size)] = _font
        else:
            _font = pygame.font.Font(self._text_font, self._text_size)

        _font.set_bold(self.text_bold)
        _font.set_italic(self.text_italic)
        _font.set_underline(self.text_underline)

        if not _internals.is_unicode_string(self.text):
            # Pygame wants latin-1 encoding here for character strings
            _text = byte2unicode(self.text).encode('latin-1')
        else:
            _text = self.text
        surface = self.render_textrect(self.format_block(_text),
                                       _font, rect, self.text_colour,
                                       self.background_colour,
                                       self.text_justification)
        return surface

    # The following code is taken from the word-wrapped text display module by
    # David Clark (http://www.pygame.org/pcr/text_rect/index.php).
    def render_textrect(self, string, font, rect, text_colour,
                        background_colour, justification=0):
        """Return a surface with reformatted string.

        The given text string is reformatted to fit within the given rect,
        word-wrapping as necessary. The text will be anti-aliased.
        Returns a surface.

        Parameters
        ----------
        string : str
            text you wish to render, '\n' begins a new line
        font : pygame.Font object, optional
            Font object
        rect : bool
            rectstyle giving the size of requested surface
        text_colour : (int, int, int)
            text colour
        background_colour : (int, int, int)
            background colour
        justification : int, optional
            0 (Left), 1 (center), 2 (right) (int) (default = 0)

        """

        final_lines = []
        requested_lines = string.splitlines()

        # Create a series of lines that will fit on the provided
        # rect.
        for requested_line in requested_lines:
            if font.size(requested_line)[0] > rect.width:
                # Start a new line
                accumulated_line = ""
                for word in requested_line.split(' '):
                    if not self._do_not_trim_words:
                        while font.size(word)[0] >= rect.width:
                            word = word[:-2] + '~'
                    elif font.size(word)[0] >= rect.width:
                            raise Exception("The word " + word +
                                " is too long to fit in the rect passed.")

                    if len(accumulated_line) > 0:
                        test_line = accumulated_line + " " + word
                    else:
                        test_line = word

                    # Build the line if the words fit.
                    if font.size(test_line)[0] < rect.width:
                        accumulated_line = test_line
                    else:
                        if len(accumulated_line) > 0:
                            final_lines.append(accumulated_line)
                        accumulated_line = word

                final_lines.append(accumulated_line)
            else:
                final_lines.append(requested_line)

        # Let's try to write the text out on the surface.
        surface = pygame.surface.Surface(rect.size,
                                         pygame.SRCALPHA).convert_alpha()
        if background_colour is not None:
            surface.fill(background_colour)
        accumulated_height = 0
        for line in final_lines:
            # Changed from >= which led to crashes sometimes!
            if accumulated_height + font.size(line)[1] > rect.height:
                raise Exception(
                    "Once word-wrapped," +
                    "the text string was too tall to fit in the rect.")
            if line != "":
                tempsurface = font.render(line, 1, text_colour)
                if justification == 0:
                    surface.blit(tempsurface, (0, accumulated_height))
                elif justification == 1:
                    surface.blit(tempsurface,
                                 ((rect.width - tempsurface.get_width()) // 2,
                                  accumulated_height))
                elif justification == 2:
                    surface.blit(tempsurface,
                                 (rect.width - tempsurface.get_width(),
                                  accumulated_height))
                else:
                    raise Exception("Invalid justification argument: " +
                                    str(justification))
            accumulated_height += font.size(line)[1]
        return surface

    def format_block(self, block):
        """Format the given block of text.

        This function is trimming leading and trailing
        empty lines and any leading whitespace that is common to all lines.

        Parameters
        ----------
        block : str
            block of text to be formatted

        """

        # Separate block into lines
        #lines = str(block).split('\n')
        lines = block.split('\n')

        # Remove leading/trailing empty lines
        while lines and not lines[0]:
            del lines[0]
        while lines and not lines[-1]:
            del lines[-1]

        # Look at first line to see how much indentation to trim
        try:
            ws = re.match(r'\s*', lines[0]).group(0)
        except:
            ws = None
        if ws:
            lines = [x.replace(ws, '', 1) for x in lines]

        # Remove leading/trailing blank lines (after leading ws removal)
        # We do this again in case there were pure-whitespace lines
        while lines and not lines[0]:
            del lines[0]
        while lines and not lines[-1]:
            del lines[-1]
        return '\n'.join(lines) + '\n'
    # End of code taken from the word-wrapped text display module


    @staticmethod
    def _demo(exp=None):
        if exp is None:
            from .. import control
            control.set_develop_mode(True)
            control.defaults.event_logging = 0
            exp_ = control.initialize()
        textbox = TextBox("Line one.\nLine two.\nLine three.", size=(100, 100))
        textbox.present()
        if exp is None:
            exp_.clock.wait(1000)



"""
A marker output.

This module contains a class implementing a marker output

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


from . import defaults
from  ._input_output  import Output
from ..misc._timer import get_time
from .. import _internals

class MarkerOutput(Output):
    """A class implementing a marker output."""

    def __init__(self, interface, default_code=None, default_duration=None):
        """Create a marker output.

        If a default code is specified, it will automatically be applied when
        send() is called without a code.
        If a default duration is specified, a 0 is sent automatically after
        the specified duration when send() is called without a duration.

        Parameters
        ----------
        interface : ``expyriment.io.SerialPort`` or ``expyriment.io.ParallelPort``
            the interface to use
        default_code : int, optional
            default code
        default_duration : int, optional
            default duration (in ms) for sending 0 after a code

        Notes
        -----
        EEG/MEG systems: if the system is receiving the markers on a parallel
        port, the duration between sending a code an the subsequent 0 should
        be at least 1000/samplerate!

        """

        Output.__init__(self)
        self._interface = interface
        if default_code is not None:
            self._default_code = default_code
        else:
            self._default_code = defaults.markeroutput_default_code
        if default_duration is not None:
            self._default_duration = default_duration
        else:
            self._default_duration = defaults.markeroutput_default_duration

    @property
    def interface(self):
        """Getter for interface"""
        return self._interface

    @property
    def default_code(self):
        """Getter for default_code"""
        return self._default_code

    @default_code.setter
    def default_code(self, value):
        """Getter for default_code"""
        self._default_code = value

    @property
    def default_duration(self):
        """Getter for default_duration"""
        return self._default_duration

    @default_duration.setter
    def default_duration(self, value):
        """Getter for default_duration"""
        self._default_duration = value


    def send(self, code=None, duration=None):
        """Send a marker.

        This sends a marker via the specified interface.
        If a duration is given, a 0 will be sent automatically after each
        code.

        Parameters
        ----------
        code : int, optional
            a specific code
        duration : int, optional
            duration (in ms) for sending a 0 after a code

        Notes
        -----
        For EEG/MEG systems: if the system is receiving the markers on a
        parallel port, the duration between sending a code an the subsequent 0
        should be at least 1000/samplerate!

        """

        if not code:
            code = self.default_code
        if not duration:
            duration = self.default_duration
        self._interface.send(code)
        if duration:
            start = get_time()
            while (get_time() - start) * 1000 < duration:
                pass
            self._interface.send(0)
        if self._logging:
            _internals.active_exp._event_file_log(
                                        "MarkerOutput,sent,{0}".format(code))



"""Data Preprocessing Module.

This module contains several classes and functions that help
to handle, preprocessing and aggregate Expyriment data files.

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''

import os as _os
try:
    import locale as _locale
except ImportError:
    _locale = None  # Does not exist on Android
import sys as _sys
from types import ModuleType
from glob import glob as _glob
from copy import copy as _copy
import codecs as _codecs
import re as _re
try:
    import numpy as _np
except:
    _np = None
from ...misc import unicode2byte as _unicode2str
from ...misc import byte2unicode as _str2unicode
from ...misc import py2py3_sort_array as _py2py3_sort_array


def read_datafile(filename, only_header_and_variable_names=False, encoding=None,
                  read_variables=None):
    """Read an Expyriment data file.

    Returns the data, the variable names, the subject info & the comments:

    Parameters
    ----------
    filename : str
        name (fullpath) of the Expyriment data file
    only_header_and_variable_names : bool, optional
        if True the function reads only the header and variable names
        (default=False)
    encoding : str, optional
        the encoding with which the contents of the file will be read
    read_variables : array of str, optional
        array of variable names, read only the specified variables

    Returns
    -------
    data : list of list
        data array
    variables : list of str
        variable names list
    subject_info : dict
        dictionary with subject information (incl. date and between
        subject factors)
    comments : str
        string with remaining comments

    """

    delimiter = ","
    variables = None
    subject_info = {}
    comments = ""
    data = []

    if encoding is None:
        with open(filename, 'r') as fl:
            first_line = fl.readline()
            encoding = _re.findall("coding[:=]\s*([-\w.]+)", first_line)
            if encoding == []:
                second_line = fl.readline()
                encoding = _re.findall("coding[:=]\s*([-\w.]+)",
                                       second_line)
                if encoding == []:
                    encoding = [None]
    else:
        encoding = [encoding]

    read_in_columns = None
    fl = _codecs.open(filename, 'rb', encoding[0], errors='replace')
    for ln in fl:
        # parse infos
        ln = _str2unicode(ln.strip())
        if not(ln.startswith("#")):
            if variables is None:
                variables = ln.split(delimiter)
                if only_header_and_variable_names:
                    break
                if read_variables is not None:
                    read_in_columns = [variables.index(x) for x in read_variables]
                    variables = [variables[x] for x in read_in_columns]
            else:
                row =ln.split(delimiter)
                if read_in_columns is not None:
                    row = [row[x] for x in read_in_columns]
                data.append(row)
        else:
            if ln.startswith("#s"):
                ln = ln.replace("#s", "")
                tmp = ln.replace("=", ":")
                tmp = tmp.split(":")
                if len(tmp) == 2:
                    subject_info[tmp[0].strip()] = tmp[1].strip()
                else:
                    subject_info["#s{0}".format(len(subject_info))] = ln.strip()
            elif ln.startswith("#date:"):
                ln = ln.replace("#date:", "")
                subject_info["date"] = ln.strip()
            else:
                comments = comments + "\n" + ln
    fl.close()
    # strip variables
    variables = [x.strip() for x in variables]
    return data, variables, subject_info, comments


def write_csv_file(filename, data, varnames=None, delimiter=','):
    """Write 2D data array to csv file.

    Parameters
    ----------
    filename : str
        name (fullpath) of the data file
    data : list of list
        2D array with data (list of list)
    varnames : list of str, optional
        array of strings representing variable names
    delimiter : str, optional
        delimiter character (default=",")

    """

    _sys.stdout.write("write file: {0}".format(filename))
    try:
        _locale_enc = _locale.getdefaultlocale()[1]
    except:
        _locale_enc = "UTF-8"
    with open(filename, 'wb') as f:
        header = "# -*- coding: {0} -*-\n".format(_locale_enc)
        f.write(_unicode2str(header))
        if varnames is not None:
            for c, v in enumerate(varnames):
                if c > 0:
                    f.write(_unicode2str(delimiter))
                f.write(_unicode2str(v))
            f.write(_unicode2str("\n"))
        cnt = 0
        for row in data:
            for c, v in enumerate(row):
                if c > 0:
                    f.write(_unicode2str(delimiter))
                f.write(_unicode2str(v))
                cnt += 1
            f.write(_unicode2str("\n"))

    print(" ({0} cells in {1} rows)".format(cnt, len(data)))


def write_concatenated_data(data_folder, file_name, output_file=None,
                            delimiter=',', to_R_data_frame=False,
                            names_comprise_glob_pattern =False):
    """Concatenate data and write it to a csv file.

    All files that start with this name will be considered for the
    analysis (cf. aggregator.data_files)

    Notes
    -----
    The function is useful to combine the experimental data and prepare for
    further processing with other software.
    It basically wraps Aggregator.write_concatenated_data.

    Parameters
    ----------
    data_folder : str
        folder which contains of the data (str)
    file_name : str
        name of the files; all files that start with this name will
        be considered
    output_file : str, optional
        name of data output file; if no specified data will the save
        to {file_name}.csv
    delimiter : str, optional
        delimiter character (default=",")
    to_R_data_frame: bool, optional
        if True, data will be converted to a R data frame that is saved
        in a RDS file
    names_comprise_glob_pattern : boolean, optional
        if True, data_folder and file_name are processed as glob pattern
        with wildcards such as "*" or "?"

    """

    if to_R_data_frame:
        return Aggregator(data_folder=data_folder, file_name=file_name,
                          names_comprise_glob_pattern=names_comprise_glob_pattern)\
        .write_concatenated_data_to_R_data_frame(output_file=output_file)
    else:
        return Aggregator(data_folder=data_folder, file_name=file_name,
                          names_comprise_glob_pattern=names_comprise_glob_pattern)\
        .write_concatenated_data(output_file=output_file, delimiter=delimiter)



def get_experiment_duration(event_filename):
    """Extracts the experiment duration from event file and returns the time in
    minutes.

    Parameters
    ----------
    info_filename : str
        name (fullpath) of the Expyriment event file

    Returns
    -------
    minutes : float
        experiment duration in minutes

    """

    data, _, _, _ = read_datafile(event_filename)

    start = end = None
    for r in data:
        if r[1] == "Experiment":
            if r[2]=="started":
                start = int(r[0])
            elif r[2]=="ended":
                stop = int(r[0])

    sec = (stop-start) / 1000.0
    return sec / 60.0


class Aggregator(object):
    """A class implementing a tool to aggregate Expyriment data.

    This class is used to handle the multiple data files of a Experiment
    and process (i.e, aggregate) the data for further analysis

    Examples
    --------
    This tool helps, for instance, to aggregate your data for certain combinations
    of independent variables. E.g., data of a numerical magnitude judgement
    experiment. The code below makes a file with mean and median RTs and a
    second file with the errors and the number of trials

    >>> from expyriment.misc import data_preprocessing
    >>> agg = data_preprocessing.Aggregator(data_folder= "./mydata/",
    >>>                                     file_name = "MagnitudeJudgements")
    >>> agg.set_computed_variables(["parity = target_number % 2",
    >>>                             "size = target_number > 65"])
    >>> agg.set_independent_variables(["hand", "size" , "parity"])
    >>>
    >>> agg.set_exclusions(["trial_counter < 0",
    >>>                     "error != 0",
    >>>                     "RT < 2*std",
    >>>                     "RT > 2*std" # remove depending std in iv factor
    >>>                                  # combination for each subject
    >>>                    ])
    >>> agg.set_dependent_variables(["mean(RT)", "median(RT)"])
    >>> agg.aggregate(output_file="rts.csv")
    >>>
    >>> agg.set_exclusions(["trial_counter < 0"])
    >>> agg.set_dependent_variables(["sum(error)", "n_trials"])
    >>> agg.aggregate(output_file="errors.csv")

    """

    _relations = ["==", "!=", ">", "<", ">=", "<=", "=>", "<="]
    _operations = ["+", "-", "*", "/", "%"]
    _dv_functions = ["mean", "median", "sum", "std", "n_trials"]

    _default_suffix = ".xpd"

    def __init__(self, data_folder, file_name, suffix=_default_suffix,
                 read_variables=None, names_comprise_glob_pattern=False):
        """Create an aggregator.

        Parameters
        ----------
        data_folder :str
            folder which contains the data
        file_name : str
            name of the files, all files that start with this name will
            be considered for the analysis (cf. aggregator.data_files)
        suffix : str, optional
            if specified only files that end with this particular
            suffix will be considered (default=.xpd)
        read_variables : array of str, optional
            array of variable names, read only the specified variables
        names_comprise_glob_pattern : boolean, optional
            if True, data_folder and file_name are processed as glob pattern
            with wildcards such as "*" or "?"; the suffix parameter will
            be ignored

        """

        if not isinstance(_np, ModuleType):
            message = """Aggregator can not be initialized.
The Python package 'Numpy' is not installed."""
            raise ImportError(message)

        _version = _np.version.version.split(".")
        if not int(_version[0]) == 1 and int(_version[1]) < 6:
            raise ImportError("Expyriment {0} ".format(__version__) +
                              "is not compatible with Numpy {0}.".format(
                                  _np.version.version) +
                              "\nPlease install Numpy 1.6 or higher.")

        print("** Expyriment Data Preprocessor **")
        self.reset(data_folder, file_name, suffix=suffix,
                   variables=read_variables,
                   names_comprise_glob_pattern=names_comprise_glob_pattern)

    def __str__(self):
        """Getter for the current design as text string."""
        design_str = "Data\n"
        design_str = design_str + u"- file name: " + self._file_name + "\n"
        design_str = design_str + u"- folder: " + self._data_folder + "\n"
        design_str = design_str + u"- {0} subject_data sets\n".format(
            len(self._data_files))
        design_str = design_str + u"- {0} variables: {1}\n".format(
            len(self.variables), self.variables)
        design_str = design_str + u"- recoded variables: {0}\n".format(
            self._recode_txt)
        design_str = design_str + u"- computed variables: {0}\n".format(
            self._computes_txt)
        design_str = design_str + u"Design\n"
        design_str = design_str + u"- independent Variables: {0}\n".format(
            self._iv_txt)
        design_str = design_str + u"- dependent Variables: {0}\n".format(
            self._dv_txt)
        design_str = design_str + u"- exclude: {0}\n".format(
            self._exclusions_txt)
        return design_str

    def _parse_syntax(self, syntax, throw_exception):
        """Preprocess relation and operation syntax.

        Returns relation array.

        """

        rels_ops = _copy(self._relations)
        rels_ops.extend(self._operations)
        found = None
        for ro in rels_ops:
            if syntax.find(ro) > 0:
                found = ro
                break
        if found is None:
            if throw_exception:
                raise RuntimeError(u"Incorrect syntax: '{0}'".format(syntax))
            else:
                return None
        else:
            syntax = syntax.split(found)
            var_id = self._get_variable_id(syntax[0].strip(), True)
            return [var_id, found, syntax[1].strip()]

    def _get_variable_id(self, variables, throw_exception=False):
        for cnt, v in enumerate(self.variables):
            if variables == v:
                return cnt
        if (throw_exception):
            raise RuntimeError(u"Unknown variable name '{0}'".format(
                variables))
        return None

    def _add_independent_variable(self, variable):
        var_id = self._get_variable_id(variable, True)
        self._iv.append(var_id)

    def _add_dependent_variable(self, variable):
        if variable == "n_trials":
            self._dv.append([variable, 0])
        else:
            tmp = variable.replace(")", "").split("(")
            dv_fnc = tmp[0].strip()
            try:
                dv_txt = tmp[1].strip()
            except:
                raise RuntimeError(
                    u"Incorrect syntax for DV: '{0}'".format(variable))
            var_id = self._get_variable_id(dv_txt, True)
            if dv_fnc in self._dv_functions:
                self._dv.append([dv_fnc, var_id])
            else:
                raise RuntimeError(
                    u"Unknown function for dependent variable:" +
                    " '{0}'".format(dv_fnc))

    def _add_compute_variable(self, compute_syntax):
        """Add a new variable to be computed."""

        tmp = compute_syntax.replace("==", "@@")  # avoid confusion = & ==
        tmp = tmp.replace("!=", "##")  # avoid confusion = & ==
        tmp = tmp.split("=")
        variable_name = tmp[0].strip()
        try:
            syntax = tmp[1].strip()
            syntax = syntax.replace("@@", "==")
            syntax = syntax.replace("##", "==")
        except:
            raise RuntimeError(u"Incorrect compute syntax: '{0}'".format(
                compute_syntax))

        variable_def = self._parse_syntax(syntax, throw_exception=True)
        if variable_def is None:
            variable_def = self._parse_operation(syntax, throw_exception=True)
        if self._get_variable_id(variable_name) is not None:
            raise RuntimeError(u"Variable already defined '{0}'".format(
                variable_name))
        else:
            self._variables.append(variable_name)
            self._computes.append([variable_name, variable_def])

    def _add_exclusion(self, relation_syntax):
        """Add an exclusion."""

        relation = self._parse_syntax(relation_syntax, throw_exception=True)
        if relation[1] in self._relations:
            self._exclusions.append(relation)
        else:
            raise RuntimeError(u"Incorrect exclusion syntax: '{0}'".format(
                relation_syntax))

    def _add_variable_recoding(self, recode_syntax):
        """Add a new variable recoding rule."""

        error = False
        tmp = recode_syntax.split(":")
        if len(tmp) == 2:
            var_id = self._get_variable_id(tmp[0].strip(), True)
            excl_array = []
            for rule in tmp[1].split(","):
                rule = rule.split("=")
                if len(rule) == 2:
                    excl_array.append([rule[0].strip(), rule[1].strip()])
                else:
                    error = True
        else:
            error = True

        if error:
            raise RuntimeError(u"Incorrect recoding syntax: '{0}'".format(
                recode_syntax))
        else:
            self._recode.append([var_id, excl_array])

    def _find_idx(self, data, column_id, relation, value):
        """Find the indices of elements in a data column.

        Notes
        -----
        It compares of column elements with a value or the elements of a second
        column, if value is a name of variable.
        The method deals with numerical and string comparisons and throws an
        exception for invalid string comparisons.

        Parameters
        ----------
        data : numpy.array
            the data
        column_id : int
            id of column to compare
        relation : str
            relation as string.  possible relations:
            "==", "!=", ">", "<", ">=", "<=", "=>", "<="
        value : numeric or string
            value to find or a variable name

        """

        # is value a variable name
        second_var_id = self._get_variable_id(value, False)

        # _add_exclusion
        try:
            col = _np.float64(data[:, column_id])
        except:
            # handling strings
            col = data[:, column_id]
        try:
            if second_var_id is not None:
                val = _np.float64(data[:, second_var_id])
            else:
                val = _np.float64(value)
        except:
            # handling strings
            if second_var_id is not None:
                val = data[:, second_var_id]
            else:
                val = value

        if value.endswith("std") and (value.find("*") > 0):
            # remove relative depending std
            tmp = value.split("*")
            fac = float(tmp[0])

            mean_stds = self._dv_mean_std(data, column_id)
            idx = []
            if relation not in [">", "<", "=>", ">=", "=<", "<="]:
                raise RuntimeError(u"Incorrect syntax for " +
                                   u"exception: '{0} {1}'".format(
                                       relation, value))
            for cnt, row in enumerate(data):
                #find name of combination
                combi_str = self.variables[column_id]
                for iv in self._iv:
                    _row_data = row[iv]
                    combi_str = combi_str + "_" + \
                        u"{0}{1}".format(self.variables[iv],
                                        _row_data)
                deviation = float(row[column_id]) - mean_stds[combi_str][0]
                if (relation == ">" and
                    deviation > fac * mean_stds[combi_str][1]) or \
                   (relation == "=>" or relation == ">=" and
                    deviation >= fac * mean_stds[combi_str][1]) or \
                   (relation == "<" and
                    deviation < -fac * mean_stds[combi_str][1]) or \
                   (relation == "=<" or relation == "<=" and
                        deviation <= -fac * mean_stds[combi_str][1]):
                        idx.append(cnt)
            return idx
        else:
            if relation == "!=":
                comp = (col != val)
            elif relation == "==":
                comp = (col == val)
            elif relation == "<":
                comp = (col < val)
            elif relation == ">":
                comp = (col > val)
            elif relation == "=<" or relation == "<=":
                comp = (col <= val)
            elif relation == "=>" or relation == ">=":
                comp = (col >= val)
            else:
                comp = None  # should never occur
            if isinstance(comp, bool):
                raise RuntimeError(
                    u"Incorrect syntax for " + u"exception: '{0} {1}'".format(
                        relation, value))
            return _np.flatnonzero(comp)

    def _dv_mean_std(self, data, column_dv_id):
        """ returns dict with std for iv_combinations """
        # get all iv values
        iv_values = []
        for iv in self._iv:
            tmp = list(set(data[:, iv]))
            _py2py3_sort_array(tmp)
            iv_values.append(tmp)

        new_variable_names, combinations = self._get_new_variables(iv_values)
        if len(combinations) == 0:
            combinations = ["total"]
        result = {}
        for cnt, fac_cmb in enumerate(combinations):
            if fac_cmb == "total":
                idx = list(range(0, data.shape[0]))
            else:
                # find idx of combinations
                idx = None
                for c, iv in enumerate(self._iv):
                    tmp = _np.array(data[:, iv] == fac_cmb[c])
                    if idx is None:
                        idx = tmp.copy()
                    else:
                        idx = idx & tmp
            # calc std over idx
            if len(idx) > 0:
                result[new_variable_names[cnt+1]] = [
                    _np.mean(_np.float64(data[idx, column_dv_id])),
                    _np.std(_np.float64(data[idx, column_dv_id]))]
                    # ignore first new var name, which is subject_id
        return result

    def _get_new_variables(self, iv_values):
        """Return the new variables names and factor_combinations.

        Requires the values for all independent variables iv_values: 2d array.
        Adds furthermore the defined the subject variables.

        """

        def increase_combination(comb, maxima, pos=None):
            """Recursive helper function.

            Returns None if end reached.

            """

            if pos is None:
                pos = len(comb) - 1
            comb[pos] += 1  # increase last position
            if comb[pos] > maxima[pos]:
                if pos <= 0:  # end reached
                    return None
                else:
                    for x in range(pos, len(comb)):  # set to zero & all pos. behind
                        comb[x] = 0
                    return increase_combination(comb, maxima, pos - 1)  # increase position before
            else:
                return comb

        # calc n levels
        n_levels = []
        for x in iv_values:
            n_levels.append(len(x) - 1)

        # build new variables names
        factor_combinations = []
        names = []
        if len(iv_values) > 0:
            tmp_comb = _np.zeros(len(self._iv), dtype=int)
            while tmp_comb is not None:
                txt = ""
                comb_values = []
                for c, x in enumerate(tmp_comb):
                    comb_values.append(iv_values[c][x])
                    if len(txt) > 0:
                        txt = txt + "_"
                    txt = txt + u"{0}{1}".format(self.variables[self._iv[c]],
                                comb_values[-1])
                names.append(txt)
                factor_combinations.append(comb_values)
                tmp_comb = increase_combination(tmp_comb, n_levels)

        new_variable_names = ["subject_id"]
        for sv in self.subject_variables:
            new_variable_names.append(u"{0}".format(sv))

        for dv in self._dv:
            if dv[0] == "n_trials":
                dv_txt = "ntr"
            else:
                dv_txt = self.variables[dv[1]]
            if len(names) > 0:
                for n in names:
                    new_variable_names.append(u"{0}_{1}".format(dv_txt, n))
            else:
                new_variable_names.append(u"{0}_total".format(dv_txt))

        return new_variable_names, factor_combinations

    def reset(self, data_folder, file_name, suffix=_default_suffix,
              variables=None, names_comprise_glob_pattern=False):
        """Reset the aggregator class and clear design.

        Parameters
        ----------
        data_folder : str
            folder which contains the data
        file_name : str
            name of the files. All files that start with this name
            will be considered for the analysis (cf. aggregator.data_files)
        suffix : str, optional
            if specified only files that end with this particular suffix
            will be considered (default=.xpd)
        variables : array of str, optional
            array of variable names, process only the specified variables
        names_comprise_glob_pattern : boolean, optional
            if True, data_folder and file_name are processed as glob pattern
            with wildcards such as "*" or "?". The suffix parameter will
            be ignored.

        """

        self._data_folder = data_folder
        self._file_name = file_name
        self._data_files = []
        self._variables = []
        self._dv = []
        self._dv_txt = []
        self._iv = []
        self._iv_txt = []
        self._exclusions = []
        self._exclusions_txt = []
        self._computes = []
        self._computes_txt = []
        self._recode_txt = []
        self._recode = []
        self._subject_variables = []
        self._last_data = []
        self._added_data = []
        self._added_variables = []
        self._suffix = suffix

        if names_comprise_glob_pattern:
            files = _glob(_os.path.join(data_folder, file_name))
        else:
            files = []
            for flname in _os.listdir(_os.path.dirname(
                    data_folder + _os.path.sep)):
                if flname.endswith(suffix) and flname.startswith(file_name):
                    files.append(_os.path.join(data_folder, flname))

        for flname in files:
                _data, vnames, _subject_info, _comments = \
                    read_datafile(flname, read_variables=variables)
                if len(self._variables) < 1:
                    self._variables = vnames
                else:
                    if vnames != self._variables:
                        message = u"Different variables in ".format(flname)
                        message = message + u"\n{0}".format(vnames)
                        message = message + u"\ninstead of\n{0}".format(
                            self._variables)
                        raise RuntimeError(message)
                self._data_files.append(flname)

        if len(self._data_files) < 1:
            raise Exception(u"No data files found")

        print(u"found {0} subject_data sets".format(len(self._data_files)))
        print(u"found {0} variables: {1}".format(len(self._variables),
                                                [x for x in self._variables]))


    @property
    def data_files(self):
        """Getter for data_files.

        The list of the data files considered for the analysis.

        """

        return self._data_files

    @property
    def variables(self):
        """Getter for variables.

        The specified variables including the new computer variables and
        between subject variables and added variables.

        """

        variables = _copy(self._variables)
        variables.extend(self._subject_variables)
        variables.extend(self._added_variables)
        return variables

    @property
    def added_variables(self):
        """Getter for added variables."""

        return self._added_variables

    @property
    def computed_variables(self):
        """Getter for computed variables."""

        return self._computes_txt

    @property
    def variable_recodings(self):
        """Getter for variable recodings."""

        return self._recode_txt

    @property
    def subject_variables(self):
        """Getter for subject variable."""

        return self._subject_variables

    @property
    def exclusions(self):
        """Getter for exclusions."""

        return self._exclusions_txt

    @property
    def dependent_variables(self):
        """Getter for dependent variables."""

        return self._dv_txt

    @property
    def independent_variables(self):
        """Getter for independent_variables."""

        return self._iv_txt

    def get_data(self, filename, recode_variables=True,
                 compute_new_variables=True, exclude_trials=True):
        """Read data from from a single Expyriment data file.

        Notes
        -----
        The function can be only applied on data of aggregator.data_files,
        that is, on the files in the defined data folder that start with
        the experiment name. According to the defined design, the result
        contains recoded data together with the new computed variables, and the
        subject variables from the headers of the Expyriment data files.

        Parameters
        ----------
        filename : str
            name of the Expyriment data file
        recode_variables : bool, optional
            set to False if defined variable recodings should not be applied
            (default=True)
        compute_new_variables : bool, optional
            set to False if new defined variables should not be computed
            (default=True)
        exclude_trials : bool, optional
            set to False if exclusion rules should not be applied
            (default=True)

        Returns
        -------
        data : numpy.array
        var_names : list
            list of variable names
        info : str
            subject info
        comment : str
            comments in data

        """

        # check filename
        if filename not in self._data_files:
            raise RuntimeError(u"'{0}' is not in the data list\n".format(
                filename))

        data, _vnames, subject_info, comments = \
            read_datafile(filename)
        print(u"   reading {0}".format(filename))

        if recode_variables:
            for var_id, recoding in self._recode:
                for old, new in recoding:
                    for row in range(len(data)):
                        if data[row][var_id] == old:
                            data[row][var_id] = new

        data = _np.array(data, dtype='|U99')
        # compute new defined variables and append
        if compute_new_variables:
            for new_var_name, var_def in self._computes:
                if var_def[1] in self._relations:
                    # relations are true or false
                    col = _np.zeros([data.shape[0], 1], dtype=int)
                    idx = self._find_idx(data, var_def[0],
                                         var_def[1], var_def[2])
                    col[idx, 0] = 1
                else:
                    # operations
                    try:
                        a = _np.float64([data[:, var_def[0]]]).transpose()
                        second_var_id = self._get_variable_id(var_def[2],
                                                              False)
                        if second_var_id is not None:
                            b = _np.float64(
                                [data[:, second_var_id]]).transpose()
                        else:
                            b = _np.float64(var_def[2])
                    except:
                        msg = "Error while computing new variable {0}. " + \
                              "Non-number in variables of {1}"
                        msg.format(new_var_name, filename)
                        raise RuntimeError(msg)
                    if var_def[1] == "+":
                        col = a + b
                    elif var_def[1] == "-":
                        col = a - b
                    elif var_def[1] == "*":
                        col = a * b
                    elif var_def[1] == "/":
                        col = a / b
                    elif var_def[1] == "%":
                        col = a % b
                data = _np.concatenate((data, col), axis=1)

        # add subject information
        for sv in self.subject_variables:
            try:
                info = subject_info[sv]
            except:
                info = "nan"
            col = _np.array([[info for _x in range(data.shape[0])]])
            data = _np.c_[data, col.transpose()]

        # _add_exclusion trials
        if exclude_trials:
            for exl in self._exclusions:
                idx = self._find_idx(data, exl[0], exl[1], exl[2])
                if len(idx) > 0:
                    data = _np.delete(data, idx, axis=0)

        var = _copy(self._variables)
        var.extend(self._subject_variables)
        return [data, var, subject_info, comments]

    @property
    def concatenated_data(self):
        """Getter for concatenated_data.

        Notes
        -----
        Returns all data of all subjects as numpy.array and all variables
        names (including added variables). According to the defined design, the
        result contains the new computed variables and the subject variables
        from the headers of the Expyriment data files.

        If data have been loaded and no new variable or exclusion has been
        defined the concatenated_data will merely return the previous data
        without re-processing.

        Returns
        -------
        data : numpy.array
        variables : list of str

        """

        if len(self._last_data) > 0:  # data are already loaded and unchanged
            cdata = self._last_data
        else:
            cdata = None
            for flname in self._data_files:
                tmp = self.get_data(flname)[0]
                if cdata is None:
                    cdata = tmp
                else:
                    cdata = _np.concatenate((cdata, tmp), axis=0)
            self._last_data = cdata

        # append added data
        if len(self._added_variables) > 0:
            if cdata is not None:
                cdata = _np.concatenate((cdata, self._added_data), axis=1)
            else:
                cdata = self._added_data

        return [cdata, self.variables]

    def get_variable_data(self, variables):
        """Returns the column of data as numpy array.

        Parameters
        ----------
        variables : list of str
            names of the variables to be extracted

        Returns
        -------
        data : numpy.array

        """

        if not isinstance(variables, (list, tuple)):
            variables = [variables]

        cols = []
        for v in variables:
            cols.append(self._get_variable_id(v, throw_exception=True))

        data = self.concatenated_data[0]
        try:
            data = _np.float64(data[:, cols])
        except:
            data = data[:, cols]

        return data

    def add_variables(self, variable_names, data_columns):
        """Adds a new variable to the data.

        Notes
        -----
        The amount of variables and added columns must match. The added data
        must also match the number of rows. Manually added variables
        will be lost if cases will be excluded afterwards via a call of
        the method `set_exclusions`.

        Parameters
        ----------
        variable_names : str
            name of the new variable(s)
        data_columns : numpy.array
            the new data columns as numpy array

        """

        d = _np.array(data_columns)
        data_shape = _np.shape(d)
        if len(data_shape) < 2:
            d = _np.transpose([d])
            data_shape = (data_shape[0], 1)
        if not isinstance(variable_names, (list, tuple)):
            variable_names = [variable_names]

        if len(variable_names) != data_shape[1]:
            raise RuntimeError(
                "Amount of variables and added columns doesn't fit.")
        if data_shape[0] != _np.shape(self.concatenated_data[0])[0]:
            raise RuntimeError("Number of rows doesn't match.")

        self._added_variables.extend(variable_names)
        if len(self._added_data) == 0:
            self._added_data = d
        else:
            self._added_data = _np.concatenate((self._added_data, d), axis=1)
        self._last_data = []

    def write_concatenated_data(self, output_file, delimiter=','):
        """Concatenates data and writes it to a csv file.

        Parameters
        ----------
        output_file : str
            name of data output file
        delimiter : str
            delimiter character (default=",")

        """

        data = self.concatenated_data
        write_csv_file(filename=output_file, data=data[0], varnames=data[1],
                       delimiter=delimiter)


    def set_independent_variables(self, variables):
        """Set the independent variables.

        Parameters
        ----------
        variables : str or list
            the name(s) of one or more data variables (aggregator.variables)

        """

        if not isinstance(variables, (list, tuple)):
            self._iv_txt = [variables]
        else:
            self._iv_txt = variables
        self._iv = []
        for v in self._iv_txt:
            self._add_independent_variable(v)
        self._last_data = []

    def set_dependent_variables(self, dv_syntax):
        """Set dependent variables.

        Parameters
        ----------
        dv_syntax : str or list
            syntax describing the dependent variable by a function and variable,
            e.g. mean(RT)

        Notes
        -----
        Syntax::

            {function}({variable})
                {function} -- mean, median, sum, std or n_trials
                              Note: n_trials counts the number of trials
                              and does not require a variable as argument
                {variable} -- a defined data variable

        """

        if not isinstance(dv_syntax, (list, tuple)):
            self._dv_txt = [dv_syntax]
        else:
            self._dv_txt = dv_syntax
        self._dv = []
        for v in self._dv_txt:
            self._add_dependent_variable(v)
        self._last_data = []

    def set_exclusions(self, rule_syntax):
        """Set rules to exclude trials from the analysis.

        The method indicates the rows, which are ignored while reading
        the data files. It can therefore not be applied on variables that have
        been added later via `add_variables` and results in a loss of all
        manually added variables. Setting exclusions requires re-reading of
        the data files and might be therefore time consuming. Thus, call this
        method always at the beginning of your analysis script.

        Parameters
        ----------
        rule_syntax : str or list
            A string or a list of strings that represent the rules
            to exclude trials

        Notes
        -----
        Rule syntax::

            {variable} {relation} {variable/value}
                {variable}  -- a defined data variable
                {relation}  --  ==, !=, >, <, >=, <=, => or <=
                {value}     -- string or numeric

                If value is "{numeric} * std", trails are excluded in which
                the variable is below or above {numeric} standard deviations
                from the mean. The relations "==" and "!=" are not allow in
                this case. The exclusion criterion is apply for each subject
                and factor combination separately.

        """

        if not isinstance(rule_syntax, (tuple, list)):
            self._exclusions_txt = [rule_syntax]
        else:
            self._exclusions_txt = rule_syntax
        self._exclusions = []
        for r in self._exclusions_txt:
            self._add_exclusion(r)
        self._last_data = []
        self._added_data = []
        self._added_variables = []

    def set_variable_recoding(self, recoding_syntax):
        """Set syntax to recode variables.

        The method defines the variables, which will recoded.  It can not
        be applied on variables that have been added later via
        `add_variables`. Recoding variables requires re-reading of the data
        files and might be therefore time consuming.

        Parameters
        ----------
        rule_syntax : str or list
            A string or a list of strings that represent the variable
            recoding syntax

        Notes
        -----
        Recoding syntax::

           {variable}: {old_value1} = {new_value1}, {old_value2} = {new_value2},...

        """

        if not isinstance(recoding_syntax, (list, tuple)):
            self._recode_txt = [recoding_syntax]
        else:
            self._recode_txt = recoding_syntax
        self._recode = []
        for syntax in self._recode_txt:
            self._add_variable_recoding(syntax)
        self._last_data = []

    def set_subject_variables(self, variables):
        """Set subject variables to be considered for the analysis.

        The method sets the subject variables. Subject variables are between
        subject factors or other variables defines in the subject information
        section (#s) of the Expyriment data file. The method requires a
        re-reading of the data files and might be therefore time consuming.

        Parameters
        ----------
        variables : str or list
            A string or a list of strings that represent the subject
            variables

        """

        if not isinstance(variables, (list, tuple)):
            self._subject_variables = [variables]
        else:
            self._subject_variables = variables
        self._last_data = []

    def set_computed_variables(self, compute_syntax):
        """Set syntax to compute new variables.

        The method defines the variables, which will be computed. It can not
        be applied on variables that have been added manually via
        `add_variables`. The method requires a re-reading of the data files
        and might be therefore time consuming.

        Parameters
        ----------
        compute_syntax : str or list
            A string or a list of strings that represent the syntax to
            compute the new variables

        Notes
        -----
        Compute Syntax::

            {new-variable} = {variable} {relation/operation} {variable/value}
                {new-variable} -- a new not yet defined variable name
                {variable}     -- a defined data variable
                {relation}     --  ==, !=, >, <, >=, <=, => or <=
                {operation}    -- +, -, *, / or %
                {value}        -- string or numeric

        """

        if not isinstance(compute_syntax, (list, tuple)):
            self._computes_txt = [compute_syntax]
        else:
            self._computes_txt = compute_syntax

        self._computes = []
        self._variables = read_datafile(self._data_files[0],
                                        only_header_and_variable_names=True)[1]  # original variables
        for syntax in self._computes_txt:
            self._add_compute_variable(syntax)
        self._last_data = []

    def print_n_trials(self, variables):
        """Print the number of trials in the combinations of the independent
        variables.

        Notes
        -----
        The functions is for instance useful to quickly check the experimental
        design.

        Parameters
        ----------
        variables : str or list
            A string or a list of strings that represent the names of one or
            more data variables (aggregator.variables)

        """

        old_iv = self._iv
        old_dv = self._dv
        self.set_dependent_variables("n_trials")
        self.set_independent_variables(variables)
        result, varnames = self.aggregate()
        for row in result:
            print("Subject {0}".format(row[0]))
            for cnt, var in enumerate(varnames):
                if cnt > 0:
                    _row_data = row[cnt]
                    print(u"\t{0}:\t{1}".format(var[4:], _row_data))
        print("\n")
        self._dv = old_dv
        self._iv = old_iv

    def aggregate(self, output_file=None, column_subject_id=0):
        """Aggregate the data as defined by the design.

        The design will be printed and the resulting data will be return as
        numpy.array together with the variable names.

        Parameters
        ----------
        output_file : str, optional
            name of data output file. If this output_file is defined the
            function write the results as csv data file
        column_subject_id : int, optional
            data column containing the subject id (default=0)

        Returns
        -------
        result : numpy.array
        new_variable_names : list of strings

        """

        data, _variables = self.concatenated_data
        subjects = list(set(data[:, column_subject_id]))
        subjects.sort()
        # get all iv values
        iv_values = []
        for iv in self._iv:
            tmp = list(set(data[:, iv]))
            _py2py3_sort_array(tmp)
            iv_values.append(tmp)

        new_variable_names, combinations = self._get_new_variables(iv_values)
        if len(combinations) == 0:
            combinations = ["total"]
        # calculate subject wise
        result = None
        for sub in subjects:
            mtx = data[data[:, column_subject_id] == sub, :]
            row = [sub]
            # subject info
            for sv in self.subject_variables:
                row.append(mtx[0, self._get_variable_id(sv)])
            for dv in self._dv:
                for fac_cmb in combinations:
                    if fac_cmb == "total":
                        idx = list(range(0, mtx.shape[0]))
                    else:
                        # find idx of combinations
                        idx = None
                        for c, iv in enumerate(self._iv):
                            tmp = _np.array(mtx[:, iv] == fac_cmb[c])
                            if idx is None:
                                idx = tmp.copy()
                            else:
                                idx = idx & tmp
                    # calc mean over idx
                    if len(idx) > 0:
                        values = mtx[idx, dv[1]]
                        if dv[0] == "median":
                            row.append(_np.median(_np.float64(values)))
                        elif dv[0] == "mean":
                            row.append(_np.mean(_np.float64(values)))
                        elif dv[0] == "sum":
                            row.append(_np.sum(_np.float64(values)))
                        elif dv[0] == "std":
                            row.append(_np.std(_np.float64(values)))
                        elif dv[0] == "n_trials":
                            row.append(values.shape[0])
                        else:
                            row.append(_np.NaN)
                    else:
                        row.append(_np.NaN)
            if result is None:
                result = _np.array([row], dtype='|U99')
            else:
                result = _np.r_[result, [row]]

        if output_file is not None:
            write_csv_file(output_file, result, new_variable_names)

        return result, new_variable_names



#!/usr/bin/env python
"""Make rst files for the expyriment API reference"""
from __future__ import absolute_import, print_function
import __future__
import builtins

import os
import sys
import inspect

p = os.path.abspath(os.path.join(os.path.split(sys.argv[0])[0], '..', '..'))
#sys.path.insert(0, p)

import expyriment
import expyriment.io.extras
import expyriment.design.extras
import expyriment.stimuli.extras
import expyriment.misc.extras


exclude = dir(builtins) + dir(__future__) + ['builtins', 'ModuleType',
                                             'True', 'False']

def inspect_members(item):
    members = inspect.getmembers(eval(item))
    modules = []
    classes = []
    methods = []
    functions = []
    attributes = []
    for member in members:
        if member[0][0:1] != '_':
            if inspect.ismodule(member[1]):
                modules.append(member)
            elif inspect.isclass(member[1]):
                classes.append(member)
            elif inspect.isfunction(member[1]):
                functions.append(member)
            elif inspect.ismethod(member[1]):
                methods.append(member)
            else:
                attributes.append(member)

    return modules, classes, methods, functions, attributes

def heading(txt, t="="):
    return txt + "\n" + len(txt)*t + "\n"


def create_class_rst(class_name):
    with open(class_name + ".rst", 'w') as fl:
        fl.write(heading(class_name))
        fl.write("\n.. autoclass:: " + class_name +"\n")
        fl.write("   :members:\n")
        fl.write("   :inherited-members:\n")
        fl.write("\n   .. automethod:: " + class_name + ".__init__\n")

def create_module_rst(mod_name, no_members=False):
    with open(mod_name + ".rst", 'w') as fl:
        fl.write(heading(mod_name))
        fl.write("\n.. automodule:: " + mod_name + "\n")
        #fl.write("   :members:\n")
        #fl.write("   :undoc-members:\n")
        #fl.write("   :show-inheritance:\n")
        #fl.write("   :inherited-members:\n")

        modules, classes, methods, functions, attributes = inspect_members(mod_name)
        if len(attributes)>0:
            fl.write(heading("\nAttributes", "-"))

            for att in attributes:
                if att[0] not in exclude:
                    att = mod_name + "." + att[0]
                    fl.write(".. py:data:: " + att + "\n\n")
                    #t = eval("type(" + att + ")")
                    if att.find("EXPYRIMENT_LOGO_FILE") == -1:
                        # do not write default for EXPYRIMENT_LOGO_FILE
                        v = eval("repr(" + att + ")")
                        fl.write("   default value: {0}\n\n".format(v))

        if len(modules)>0:
            fl.write(heading("\n\nModules", "-"))
            fl.write(".. toctree::\n   :maxdepth: 1\n   :titlesonly:\n")

            for m in modules:
                if m[0] not in exclude:
                    fl.write("\n   " + mod_name + "." + m[0])
                    create_module_rst(mod_name + "." + m[0])

        if len(classes)>0:
            fl.write(heading("\n\nClasses", "-"))
            fl.write(".. toctree::\n   :titlesonly:\n")

            for cl in classes:
                if cl[0] not in exclude:
                    fl.write("\n   " + mod_name + "." + cl[0])
                    create_class_rst(mod_name + "." + cl[0])

        if len(functions)>0:
            fl.write(heading("\n\nFunctions", "-"))
            for func in functions:
                if func[0] not in exclude:
                    fl.write(".. autofunction:: " + mod_name + "." + func[0] + "\n")

        fl.write("\n\n")
        #fl.write("\n\n.. "+repr(modules) + "\n")
        #fl.write(".. "+repr(classes) + "\n")
        #fl.write(".. "+repr(methods) + "\n")
        #fl.write(".. "+repr(functions) + "\n")
        #fl.write(".. "+repr(attributes) + "\n")


def create_change_log_rst():
    """Create well shaped Changelog.rst from CHANGES.md."""

    changes_md = os.path.join(p, "CHANGES.md")
    changelog_rst = "Changelog.rst"

    fl = open(changes_md, 'r')
    out = open(changelog_rst, 'w')

    out.write("""Changelog
==========

""")

    version_found = False
    for line in fl:
        if line.startswith("Version"):
            version_found = True
        if version_found:
            if line.startswith("New Feature") or line.startswith("Fixed") or\
                    line.startswith("Changes") or line.startswith("Changed"):
                out.write("\n" + line + "\n")  # additional blanklines
            elif line.startswith("--"):
                out.write(line + "\n")
            elif line.startswith("  - "):
                out.write("\n" + line)
            else:
                out.write(line)

    out.close()
    fl.close()

# main module
if __name__ == '__main__':

    with open("expyriment.rst", 'w') as fl:
        fl.write("""
expyriment
==========

.. automodule:: expyriment


Packages
--------

.. toctree::
   :maxdepth: 1
   :titlesonly:

   expyriment.control
   expyriment.design
   expyriment.io
   expyriment.misc
   expyriment.stimuli

Functions
---------
.. autofunction:: expyriment.get_version
.. autofunction:: expyriment.import_all_extras
.. autofunction:: expyriment.show_documentation

""")

    sub_modules = ["expyriment.io", "expyriment.design", "expyriment.stimuli",
               "expyriment.control", "expyriment.misc"]

    #sub_modules
    for mod_name in sub_modules:
        create_module_rst(mod_name)

    create_change_log_rst()



#!/usr/bin/env python

"""
A picture stimulus.

This module contains a class implementing a picture stimulus.

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


import os

import pygame
from .. import _internals
from ..misc import unicode2byte
from . import defaults
from ._visual import Visual


class Picture(Visual):
    """A class implementing a general picture stimulus."""

    def __init__(self, filename, position=None):
        """Create a picture.

        Parameters
        ----------
        filename : str
            filename (incl. path) of the picture file
        position :(int,int), optional
            position of the stimulus

        """

        if position is None:
            position = defaults.picture_position
        Visual.__init__(self, position, log_comment=filename)
        self._filename = filename
        if not(os.path.isfile(self._filename)):
            raise IOError(u"The picture file '{0}' does not exist".format(
                filename))

    _getter_exception_message = "Cannot set {0} if surface exists!"

    @property
    def filename(self):
        """Getter for filename."""

        return self._filename

    @filename.setter
    def filename(self, value):
        """Setter for filename."""

        if self.has_surface:
            raise AttributeError(Picture._getter_exception_message.format(
                "filename"))
        else:
            self._filename = value

    def _create_surface(self):
        """Create the surface of the stimulus."""

        # Due to a bug in handling file names in PyGame 1.9.2, we pass a file
        # handle to PyGame. See also:
        # https://github.com/expyriment/expyriment/issues/81
        with open(self._filename, 'rb') as f:
            surface = pygame.image.load(f).convert_alpha()
        if self._logging:
            _internals.active_exp._event_file_log(
                "Picture,loaded,{0}".format(self._filename), 1)
        return surface


    @staticmethod
    def _demo(exp=None):
        if exp is None:
            from .. import __file__
            from .. import control
            control.set_develop_mode(True)
            control.defaults.event_logging = 0
            exp_ = control.initialize()
        directory = os.path.dirname(__file__)
        picture = Picture(os.path.join(directory, "expyriment_logo.png"))
        picture.present()
        if exp is None:
            exp_.clock.wait(1000)
